/*global document, include, window */
(function () {
    "use strict";

    // Library dependencies
    var Application = include('springroll.Application'),
        Display = include('springroll.pixi.PixiDisplay'),
        TextureAtlas = include('PIXI.spine.core.TextureAtlas', false),
        AtlasAttachmentLoader = include('PIXI.spine.core.AtlasAttachmentLoader', false),
        BaseTexture = include('PIXI.BaseTexture'),
        SkeletonJson = include('PIXI.spine.core.SkeletonJson', false),
        Spine = include('PIXI.spine.Spine', false),
        app = new Application({ // Create a new application
            name: "MissionEarth",
            canvasId: "stage",
            captions: "captions",
            configPath: "assets/config/config.json",
            display: Display,
            displayOptions: {
                clearView: true
            },
            enableHiDPI: true,
            preload: [
                {"id": "Black", "src": "assets/images/Black.png"}
            ],
            resizeElement: "content",
            responsive: true,
            uniformResize: false,
            state: "title",
            version: "1.2.21"
        }),
        resize = function () {// RJG website doesn't enforce aspect ratio for vertical mobile devices, so we handle it here to keep things looking nice. - DDD 8-9-2017
            var frame = document.getElementById('frame'),
                height = window.innerHeight,
                width = window.innerWidth,
                newHeight = (width * 0.75) >> 0;
            
            if (height > newHeight) {
                frame.style.height = newHeight + 'px';
                frame.style.top = (((height - newHeight) / 2) >> 0) + 'px';
            } else {
                frame.style.height = '';
                frame.style.top = '';
            }
        },
        savedWidth = 0,
        savedHeight = 0;

    // Handle when app is ready to use
    app.on('init', function () {
        // Start game
        var settings = app.config.platypus,
            imageCallback = function (line, callback) {
                callback(BaseTexture.fromImage(line));
            };

        app.transition = new Spine(new SkeletonJson(new AtlasAttachmentLoader(new TextureAtlas(settings.atlases.Black, imageCallback))).readSkeletonData(settings.skeletons.Black));
        app.transition.scale.x = 6;
        app.transition.scale.y = 6;
        if (savedWidth && savedHeight) {
            app.transition.x = savedWidth / 2;
            app.transition.y = savedHeight / 2;
        }

        app.captions.data = app.config.captions;
    });

    app.on('resize', function (width, height) {
        if (app.transition) {
            app.transition.x = width / 2;
            app.transition.y = height / 2;
        } else {
            savedWidth = width;
            savedHeight = height;
        }
    });

    // Assign to the window for easy access
    window.app = app;

    // Handle frame resize to prevent bad-looking aspect ratios.
    window.addEventListener('resize', resize, false);
    window.addEventListener('orientationchange', resize, false);
    window.addEventListener('focus', resize, false);
    window.addEventListener('load', resize, false);

    // Monkey-patched work-around for SpringRoll bug: https://github.com/SpringRoll/SpringRoll/issues/148
    app.userData.___read = app.userData.read;
    app.userData.read = (function () {
        var busy = false,
            callbackWrapper = function (callback, value) {
                var next = null;

                callback(value);

                if (queue.length) {
                    next = queue.shift();
                    app.userData.___read(next.key, callbackWrapper.bind(null, next.callback));
                } else {
                    busy = false;
                }
            },
            queue = [];

        return function (key, callback) {
            if (!busy) {
                busy = true;
                app.userData.___read(key, callbackWrapper.bind(null, callback));
            } else {
                queue.push({
                    key: key,
                    callback: callback
                });
            }
        };
    }());
}());

/* global include, platypus */

(function () {
    'use strict';

    var AABB = include('platypus.AABB');

    return platypus.createComponentClass({
        id: 'AudioAura',

        properties: {
            range: 1000,
            
            sfx: '',
            credits: null
        },

        initialize: function (definition) {
            this.playEvent = this.owner.playEvent || definition.playEvent || ('sfx-' + this.sfx) || null;
            this.maxDistance = this.range * this.range;
            this.cameraPosition = AABB.setUp();
            this.audioPaused = true;
            this.volume = 0;
            this.pan = 0;
            this.active = true;

            if (this.credits && this.credits[this.playEvent]) {
                console.log(this.credits[this.playEvent]);
            }
        },

        events: {// These are messages that this component listens for
            "camera-update": function (cameraInfo) {
                this.cameraPosition.set(cameraInfo.viewport);
            },
            "handle-logic": function () {
                var cp = this.cameraPosition,
                    volume = 0,
                    distance = Math.pow(this.owner.x - cp.x, 2) + Math.pow(this.owner.y - cp.y, 2),
                    pan = 0;
                
                volume = Math.max(1 - (distance / this.maxDistance), 0);
                if (volume === 0) {
                    if (!this.audioPaused) {
                        this.owner.triggerEvent('stop-audio', this.sfx);
                        this.audioPaused = true;
                        this.volume = 0;
                        this.pan = 0;
                    }
                    return;
                } else if (this.audioPaused) {
                    this.audioPaused = false;
                    this.owner.triggerEvent(this.playEvent);
                }
                
                if (volume !== this.volume) {
                    this.volume = volume;
                    this.owner.triggerEvent('set-volume', volume, this.sfx);
                }

                if (this.owner.x > cp.right) {
                    pan = 1;
                } else if (this.owner.x < cp.left) {
                    pan = -1;
                } else {
                    pan = ((this.owner.x - cp.left) / cp.width) * 2 - 1;
                }
                if (pan !== this.pan) {
                    this.pan = pan;
                    this.owner.triggerEvent('set-pan', pan, this.sfx);
                }
            }
        }
    });
})();

/* global include, platypus */

(function () {
    'use strict';

    return platypus.createComponentClass({
        id: 'AuraBackground',

        properties: {
        },

        initialize: function (definition) {
            this.sfx = null;
            this.volume = 1;
            this.pan = 0;
            this.lastVolume = 1;
            this.lastPan = 0;
        },

        events: {// These are messages that this component listens for
            "jet-active": function () {
                if (this.sfx !== 'jetloop') {
                    if (this.sfx) {
                        this.owner.triggerEvent('stop-audio', this.sfx);
                    }
                    this.sfx = 'jetloop';
                    this.owner.triggerEvent(this.sfx, {
                        volume: this.volume,
                        pan: this.pan
                    });
                }
            },

            "mindy-active": function () {
                if (this.sfx !== 'mindyloop') {
                    if (this.sfx) {
                        this.owner.triggerEvent('stop-audio', this.sfx);
                    }
                    this.sfx = 'mindyloop';
                    this.owner.triggerEvent(this.sfx, {
                        volume: this.volume,
                        pan: this.pan
                    });
                }
            },

            "aura-volume": function (volume) {
                this.volume = 1 - volume;
            },

            "aura-pan": function (pan) {
                if (pan > 0) {
                    this.pan = 1 - pan;
                } else if (pan < 0) {
                    this.pan = 1 + pan;
                } else {
                    this.pan = 0;
                }
            },

            "aura-stopped": function () {
                this.volume = 1;
                this.pan = 0;
            },

            "handle-logic": function () {
                if (this.sfx) {
                    if (this.volume !== this.lastVolume) {
                        this.lastVolume = this.volume;
                        this.owner.triggerEvent('set-volume', this.volume, this.sfx);
                    }

                    if (this.pan !== this.lastPan) {
                        this.lastPan = this.pan;
                        this.owner.triggerEvent('set-pan', this.pan, this.sfx);
                    }
                }
            }
        }
    });
})();


/*global platypus */
(function () {
    'use strict';
    var SPEED_BASE = 0.003,
        SPEED_VARIANCE = 0.003,
        AMPLITUDE_BASE = 148,
        AMPLITUDE_VARIANCE = 64;
    
    return platypus.createComponentClass({
        
        id: 'Bee',
        properties: {
        },
        
        initialize: function () {

            this.speed  = Math.random() * SPEED_VARIANCE + SPEED_BASE;
            this.amplitude = Math.random() * AMPLITUDE_VARIANCE + AMPLITUDE_BASE;
            this.orbit = Math.random();
            if (this.orbit < 0.2) {
                this.orbit += 0.2;
            } else if (this.orbit > 0.8) {
                this.orbit -= 0.2;
            }

            if (this.owner.orbitVertical && this.orbit < 0.5) {
                this.orbit = 1 - this.orbit;
            }

            this.owner.state.set('bee-0', false);
            this.owner.state.set('bee-1', false);
            this.owner.state.set('bee-2', false);
            this.owner.state.set('bee-3', false);
            this.owner.state.set('c-w', false);

            this.position = Math.random() * (Math.PI * 2);

            this.origin = {
                "x": this.owner.x + (Math.random() * 96 - 48),
                "y": this.owner.y + (Math.random() * 96 - 48)
            };

            this.headingRight = true;
        },
        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta,
                    offsetX = 0,
                    offsetY = 0,
                    direction = this.owner.state.get('c-w') ? 1 : -1;

                this.position += delta * this.speed * direction;
                this.position = this.position % (Math.PI * 2);
                
                offsetX = Math.cos(this.position) * (this.amplitude * (1 - this.orbit));
                offsetY = Math.sin(this.position) * (this.amplitude * this.orbit);

                this.owner.x = this.origin.x + offsetX;
                this.owner.y = this.origin.y + offsetY;

                if (this.headingRight && Math.abs(this.position) < Math.PI) {
                    this.headingRight = false;
                    this.owner.triggerEvent('transform', 'horizontal');
                } else if (!this.headingRight && Math.abs(this.position) > Math.PI) {
                    this.headingRight = true;
                    this.owner.triggerEvent('transform', 'horizontal');
                }
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    var BEE_COUNT = (platypus.supports.mobile ? 3 : 7),
        BEE_TYPES = 4;
    return platypus.createComponentClass({
        
        id: 'BeeHive',
        properties: {
        },
        
        initialize: function () {
            this.bees = [];
        },
        events: {
            "adopted": function () {
                var x = 0,
                    newBee = null,
                    cwToggle = false;

                for (x = 0; x < BEE_COUNT; x++) {
                    newBee = this.owner.parent.addEntity({
                        "type": "bee",
                        "properties": {
                            "x": this.owner.x,
                            "y": this.owner.y,
                            "z": this.owner.z + 0.001,
                            "orbitVertical": cwToggle
                        }
                    });
                    newBee.state.set('bee-' + (x % BEE_TYPES), true);
                    newBee.state.set('c-w', cwToggle);
                    cwToggle = !cwToggle;
                    this.bees.push(newBee);
                }
            }
        },
        
        methods: {
            destroy: function () {
                this.bees = null;
            }
        },

        getAssetList: function (component, props, defaultProps) {
            return ['assets/images/bees.png'];
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    var BINOCULAR_CAMERA_WIDTH = 490,
        BINOCULAR_CAMERA_HEIGHT = 278;

    return platypus.createComponentClass({
        id: 'BinocularTarget',
        
        events: {
            "found": function () {
                this.owner.opacity = 1;
            },

            "camera-update": function (camera) {
                if (this.owner.opacity && (camera.viewport.width > BINOCULAR_CAMERA_WIDTH) && (camera.viewport.height > BINOCULAR_CAMERA_HEIGHT)) {
                    this.owner.parent.removeEntity(this.owner);
                }
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        id: 'BinocularTargeter',

        events: {
            "load": function () {
                this.owner.tint = 0xff4444;
            },

            "found-goal-target": function () {
                this.owner.tint = 0x44ff44;
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var SLOW_FADE = 750,
        FAST_FADE = 250,
        Vector = include('platypus.Vector'),
        remove = function (entity) {
            entity.triggerEvent('force-release');
            this.removeEntity(entity);
        },
        swapTo = function (character, data) {
            var owner = this.owner;

            if (owner.state.get('on-ground')) {
                this.swapping = character;
                owner.triggerEvent('begin-swap');
            }
        };
    
    return platypus.createComponentClass({
        id: 'CharacterSwapper',

        properties: {
            characters: null,
            preventSwap: false,
            tool: '',
            waving: false,
            freezeOnSwap: false
        },
        
        publicProperties: {
            checkpointPosition: []
        },
        
        initialize: function () {
            var characters = this.characters,
                i = characters.length;

            this.fadingOut = false;
            this.fadingIn = false;

            this.swapping = '';
            this.checkpoint = Vector.setUp(this.checkpointPosition[0] || this.owner.x, this.checkpointPosition[1] || this.owner.y);
            this.toolInstances = Array.setUp();

            if (!this.preventSwap) {
                while (i--) {
                    this.addEventListener(characters[i], swapTo.bind(this, characters[i]));
                }
            }

            //this.owner.state.set('transition-in', true);
            this.owner.opacity = 0;
            this.fadingIn = true;
            this.owner.paused = true;

            this.originalZ = this.owner.z;
            this.fadeSpeed = FAST_FADE;
            if (this.waving) {
                this.owner.state.set('waving', true);
            }

            if (this.freezeOnSwap) {
                this.frozenX = this.owner.x;
                this.frozenY = this.owner.y;
            }
        },

        events: {// These are messages that this component listens for
            "adopted": (function () {
                var addTool = function (layer, tool, dest) {
                        dest.push(layer.addEntity(tool));
                    },
                    addTools = function (layer, tools, dest) {
                        var i = tools.length;

                        while (i--) {
                            addTool(layer, tools[i], dest);
                        }
                    };

                return function () {
                    var tool = this.tool,
                        layer = this.owner.parent.parent.layers[1];

                    if (tool) {
                        if (Array.isArray(tool)) {

                            // The zeroeth (discovery tool) always appears. The movement tools (Jet's arm) are mobile only.
                            if (!platypus.supports.mobile) {
                                tool.length = 1;
                            }

                            addTools(layer, tool, this.toolInstances);
                        } else {
                            addTool(layer, tool, this.toolInstances);
                        }
                    }
                };
            }()),

            "begin-swap": function () {
                var owner = this.owner;

                owner.paused = true;
                owner.triggerEvent('stop-movement');
                this.fadingOut = true;
            },

            "handle-logic": function (tick) {
                var delta = tick.delta,
                    owner = this.owner,
                    parent = owner.parent;

                if (this.fadingOut) {
                    this.owner.opacity -= delta / this.fadeSpeed;
                    if (this.owner.opacity <= 0) {
                        this.fadingOut = false;

                        if (this.swapping) {
                            parent.addEntity({
                                type: this.swapping,
                                properties: {
                                    headingRight: owner.headingRight,
                                    locked: this.freezeOnSwap,
                                    waving: this.freezeOnSwap,
                                    x: this.frozenX || owner.x,
                                    y: this.frozenY || Math.round(owner.y / 64) * 64, // Because characters could be in the ground during logic.
                                    z: this.originalZ,
                                    paused: true,
                                    checkpointPosition: [this.checkpoint.x, this.checkpoint.y]
                                }
                            }, remove.bind(parent, owner));
                        } else if (this.killed) {
                            parent.addEntity({
                                type: owner.type,
                                properties: {
                                    headingRight: owner.headingRight,
                                    x: this.checkpoint.x,
                                    y: this.checkpoint.y,
                                    z: this.originalZ,
                                    paused: true,
                                    checkpointPosition: [this.checkpoint.x, this.checkpoint.y]
                                }
                            }, remove.bind(parent, owner));
                        }
                    }
                } else if (this.fadingIn) {
                    this.owner.opacity += delta / this.fadeSpeed;
                    if (this.owner.opacity >= 1) {
                        this.fadingIn = false;
                        if (!this.owner.locked) {
                            owner.paused = false;
                        }
                    }
                }
            },

            "kill": function (collisionData) {
                if (!this.killed) {
                    this.respawn(collisionData);
                    this.killed = true;
                }
            },

            "muddify": function (collisionData) {
                if (!this.killed) {
                    if (!platypus.game.app.voPlayer.playing) {
                        this.owner.triggerEvent('yucky'); // This sfx is in the character's voice, so we don't want to double-up if we can help it.
                    }
                    this.respawn(collisionData);
                    this.killed = true;
                }
            },

            "set-checkpoint-position": function (collision) {
                this.checkpoint.x = collision.entity.x;
                this.checkpoint.y = collision.entity.y;
            },

            "lock": function () {
                this.owner.triggerEvent('stop-movement');
                this.owner.paused = true;
            },

            "dance": function () {
                this.owner.state.set('dancing', true);
            },

            "reach": function () { // For Jet's intro.
                this.owner.state.set('waving', false);
            },

            "binoc-example": function () { // For Jet's intro.
                this.owner.state.set('looking-success', true);
                this.owner.triggerEvent('stop-movement');
                this.owner.paused = true;
            },

            "binoc-example-end": function () { // For Jet's intro.
                this.owner.state.set('looking-success', false);
                this.owner.state.set('looking-success-over', true);
            },

            "unlock-jet": function () { // For Jet's intro.
                this.owner.paused = false;
                this.owner.state.set('looking-success-over', false);
            },

            "unlock": function () {
                this.owner.paused = false;
                this.owner.state.set('waving', false);
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            respawn: function (collisionData) {
                var owner = this.owner,
                    x = 0,
                    splashCount = (platypus.supports.mobile ? 5 : 20); // Optimize for mobile
                
                collisionData.entity.triggerEvent('splash-sfx');
                for (x = 0; x < splashCount; x++) {
                    this.makeDroplet(collisionData.entity.type);
                }
                owner.z = 950;
                owner.paused = true;
                owner.triggerEvent('stop-x-movement');
                this.fadingOut = true;
                this.fadeSpeed = SLOW_FADE;
            },

            makeDroplet: function (type) {
                var newEntity = null,
                    propertiesContainer = {
                        "type": "droplet",
                        "properties": {
                            x: this.owner.x,
                            y: this.owner.y,
                            z: this.owner.z + 0.0001,
                            spawner: this.owner
                        }
                    };
                newEntity = this.owner.parent.addEntity(propertiesContainer);
                if (type === "mud") {
                    newEntity.state.set('mud', true);
                }
            },
            destroy: function () {
                var tools = this.toolInstances,
                    i = tools.length,
                    layer = this.owner.parent.parent.layers[1];

                while (i--) {
                    layer.removeEntity(tools[i]);
                }
                tools.recycle();
                this.toolInstances = null;
                this.checkpoint.recycle();
                this.checkpoint = null;
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';
    
    return platypus.createComponentClass({

        id: 'CheckPulse',

        initialize: function () {
            this.pulsing = false;
            this.pulsed = false;
        },

        events: {
            "handle-logic": function () {
                if (this.pulsing && !this.pulsed) {
                    this.owner.triggerEvent('pulse');
                }
                this.pulsed = this.pulsing;
            },
            "reset-pulsing": function () {
                this.pulsed = false;
            },
            "start-pulsing": function () {
                this.pulsing = true;
            },
            "stop-pulsing": function () {
                this.pulsing = false;
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    var RESET_RANGE = 1000,
        GRAVITY = 0.001,
        FRICTION = 0.95;
    
    return platypus.createComponentClass({
        
        id: 'Drip',
        properties: {
            "baseXV": -0.1,
            "varianceXV": -0.15,
            "baseScale": 0.15,
            "varianceScale": 0.2,
            "horizontalSpread": 8,
            "fade": false,
            "dripType": null,
            "verticalFall": 256,
            "source": null
        },
        
        initialize: function () {
           
            this.offsetX = 0;

            this.xFromSource = 0;
            this.yFromSource = 0;

            this.reset = true;
            this.resetTime = 0;

            this.owner.opacity = 0;

            this.vX = 0;
            this.vY = 0;
            this.scale = 1;

            if (this.dripType) {
                this.owner.state.set(this.dripType, true);
            }
        },
        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;

                if (this.reset) {
                    this.reset = false;
                    this.resetTime = Math.random() * RESET_RANGE;
                    this.offsetX = Math.random() * this.horizontalSpread - this.horizontalSpread / 2;

                    this.xFromSource = this.offsetX;
                    this.yFromSource = 0;
                    
                    this.vX = this.baseXV + Math.random() * this.varianceXV;
                    this.vY = 0;
                   
                    this.scale = this.baseScale + Math.random() * this.varianceScale;
                    this.owner.scaleX = this.scale;
                    this.owner.scaleY = this.scale;
                } else if (this.resetTime) {
                    this.resetTime -= delta;
                    if (this.resetTime < 0) {
                        this.resetTime = 0;
                        this.owner.opacity = 1;
                    }
                } else {
                    this.vX *= FRICTION;
                    this.vY += GRAVITY * delta;
                    this.xFromSource += this.vX * delta;
                    this.yFromSource += this.vY * delta;

                    if (this.fade) {
                        this.owner.opacity = 1 - (this.yFromSource / this.verticalFall);
                    }

                    if (this.owner.y >= this.source.y + this.verticalFall) {
                        this.reset = true;
                        this.owner.opacity = 0;
                    }
                }

                this.owner.x = this.source.x + this.xFromSource;
                this.owner.y = this.source.y + this.yFromSource;
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    return platypus.createComponentClass({
        
        id: 'DripHive',
        properties: {
            "dripCount": 13,
            "baseXV": -0.04,
            "varianceXV": -0.07,
            "baseScale": 0.125,
            "varianceScale": 0.15,
            "horizontalSpread": 8,
            "fade": false,
            "dripType": null,
            "verticalFall": 256,
            "zOffset": -0.001
        },
        
        initialize: function () {
        },
        events: {
            "adopted": function () {
                var drips = (platypus.supports.mobile ? Math.ceil(Math.sqrt(this.dripCount)) : this.dripCount),
                    x = 0;



                for (x = 0; x < drips; x++) {
                    this.owner.parent.addEntity({
                        "type": "drip",
                        "properties": {
                            "x": this.owner.x,
                            "y": this.owner.y,
                            "z": this.owner.z + this.zOffset,
                            "baseXV": this.baseXV,
                            "varianceXV": this.varianceXV,
                            "baseScale": this.baseScale,
                            "varianceScale": this.varianceScale,
                            "horizontalSpread": this.horizontalSpread,
                            "fade": this.fade,
                            "dripType": this.dripType,
                            "verticalFall": this.verticalFall,
                            "source": this.owner
                        }
                    });
                }
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    var OFFSET_VARIANCE = 500,
        OFFSET_BASE = 250,
        GOOSE_COUNT = (platypus.supports.mobile ? 3 : 6);
        
    return platypus.createComponentClass({
        
        id: 'FlockHive',
        properties: {

        },
        
        initialize: function () {
            this.count = 0;
            this.zOffset = 0.5;

            this.timer = OFFSET_BASE + OFFSET_VARIANCE * Math.random();
        },
        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;
                if (this.count < GOOSE_COUNT) {
                    this.timer -= delta;
                    if (this.timer <= 0) {
                        this.count++;
                        this.timer = OFFSET_BASE + OFFSET_VARIANCE * Math.random();

                        this.owner.parent.addEntity({
                            "type": "goose",
                            "properties": {
                                "x": this.owner.x,
                                "y": this.owner.y,
                                "z": this.owner.z - this.zOffset,
                                "source": this.owner
                            }
                        });
                    }
                }
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    var RESET_VARIANCE = 1000,
        POSITION_VARIANCE = -72,
        V_BASE = 0.02,
        V_VARIANCE = 0.06,
        SCALE_BASE = 0.125,
        SCALE_VARIANCE = 0.2,
        FLIGHT_ANGLE = Math.PI / 8,
        X_DISTANCE_BASE = 256,
        X_DISTANCE_VARIANCE = 64;
    
    return platypus.createComponentClass({
        
        id: 'Goose',
        properties: {
            "source": null
        },
        
        initialize: function () {
           
            this.xFromSource = 0;
            this.yFromSource = 0;

            this.reset = true;
            this.resetTime = 0;

            this.owner.opacity = 0;

            this.v = {
                "x": 0,
                "y": 0
            };

            this.flightVector = {
                "x": Math.cos(FLIGHT_ANGLE),
                "y": -Math.sin(FLIGHT_ANGLE)
            };

            this.xDistanceTraveled = 0;

            this.startingVX = 0;
            this.startingVY = 0;
            this.startingScale = 0;

            this.originalZ = this.owner.z;
            this.goalXDistance = X_DISTANCE_BASE + X_DISTANCE_VARIANCE * Math.random();
        },
        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta,
                    v = 0,
                    portion = 0;

                if (this.reset) {
                    this.reset = false;
                    this.resetTime = Math.random() * RESET_VARIANCE;

                    this.xDistanceTraveled = 0;
                    this.xFromSource = 0;
                    this.yFromSource = Math.random() * POSITION_VARIANCE;

                    this.startingScale = Math.random() * SCALE_VARIANCE + SCALE_BASE;
                    this.owner.scaleX = this.startingScale;
                    this.owner.scaleY = this.startingScale;

                    v =  V_VARIANCE * this.startingScale * 3 + V_BASE + Math.random() * 0.01;
                    this.startingVX = this.flightVector.x * v;
                    this.startingVY = this.flightVector.y * v;
                    this.v.x = this.startingVX;
                    this.v.y = this.startingVY;

                    this.owner.z = this.originalZ + 0.25 * this.startingScale;
                    this.goalXDistance = X_DISTANCE_BASE + X_DISTANCE_VARIANCE * Math.random();
                } else if (this.resetTime) {
                    this.resetTime -= delta;
                    if (this.resetTime < 0) {
                        this.resetTime = 0;
                        this.owner.opacity = 1;
                    }
                } else {
                    portion =  1 - (this.xDistanceTraveled / this.goalXDistance);

                    this.v.x = (0.5 * this.startingVX) + (0.5 * this.startingVX * portion);
                    this.v.y = (0.5 * this.startingVY) + (0.5 * this.startingVY * portion);
                    this.owner.scaleX = (0.5 * this.startingScale) + (0.5 * this.startingScale * portion);
                    this.owner.scaleY = (0.5 * this.startingScale) + (0.5 * this.startingScale * portion);
                    this.owner.opacity = Math.min(2 * portion, 1);

                    this.xFromSource += this.v.x * delta;
                    this.yFromSource += this.v.y * delta;
                    this.xDistanceTraveled += this.v.x * delta;

                    if (this.xDistanceTraveled >= this.goalXDistance) {
                        this.reset = true;
                        this.owner.opacity = 0;
                    }
                }

                this.owner.x = this.source.x + this.xFromSource;
                this.owner.y = this.source.y + this.yFromSource;
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());

/* global include, platypus */

(function () {
    'use strict';

    var AABB = include('platypus.AABB');

    return platypus.createComponentClass({
        id: 'HairFlipper',

        properties: {
            earlyFlip: false,
            hair: null
        },

        initialize: function (definition) {
            this.spine = null;
            this.hairIndex = 0;
        },

        events: {// These are messages that this component listens for
            "load": function () {
                var components = this.owner.components,
                    i = components.length;

                // get a reference to spine
                while (i--) {
                    if (components[i].spine) {
                        this.spine = components[i].spine;
                        break;
                    }
                }

                if (this.earlyFlip) {
                    this.hairIndex = 1 - this.hairIndex;
                    this.spine.skeleton.setAttachment('Head', this.hair[this.hairIndex]);
                }
            },

            "transform": function () {
                // We must flip the hair!
                if (this.spine) {
                    this.hairIndex = 1 - this.hairIndex;
                    this.spine.skeleton.setAttachment('Head', this.hair[this.hairIndex]);
                } else {
                    this.earlyFlip = !this.earlyFlip;
                }
            }
        }
    });
})();

/*global include, platypus */
(function () {
    'use strict';

    var BOOM_X = 192,
        BOOM_Y = -128,
        MAX_RIGHT = 512,
        MAX_LEFT = -512,
        MAX_TOP = -256,
        MAX_BOTTOM = 128,
        TRANSITION_RATE = 0.025,
        Vector = include('platypus.Vector'),
        handleLogic = function () {
            var boom = this.offset,
                last = this.last,
                lastPrimary = this.lastPrimary,
                owner = this.owner,
                position = owner.position,
                primary = this.primary,
                v = Vector.setUp(boom);
            
            boom.x += primary.position.x - lastPrimary.x;
            if (boom.x > BOOM_X) {
                boom.x = BOOM_X;
            } else if (boom.x < -BOOM_X) {
                boom.x = -BOOM_X;
            }

            lastPrimary.set(primary.position);

            position.set(primary.position).addVector(v);

            position.subtractVector(last);

            if (position.x > MAX_RIGHT) {
                position.x = (position.x - MAX_RIGHT) + (MAX_RIGHT * TRANSITION_RATE);
            } else if (position.x < MAX_LEFT) {
                position.x = (position.x - MAX_LEFT) + (MAX_LEFT * TRANSITION_RATE);
            } else {
                position.x *= TRANSITION_RATE;
            }

            if (position.y > MAX_BOTTOM) {
                position.y = (position.y - MAX_BOTTOM) + (MAX_BOTTOM * TRANSITION_RATE);
            } else if (position.y < MAX_TOP) {
                position.y = (position.y - MAX_TOP) + (MAX_TOP * TRANSITION_RATE);
            } else {
                position.y *= TRANSITION_RATE;
            }

            position.addVector(last);

            last.set(position);
        };
    
    return platypus.createComponentClass({
        id: 'HeroCamera',

        initialize: function () {
            this.primary = null;
            this.offset = Vector.setUp(0, BOOM_Y, 0);
            this.last = null;
            this.lastPrimary = null;
        },

        events: {
            "adopted": function () {
                this.owner.parent.triggerEvent('follow', {
                    mode: 'locked',
                    entity: this.owner
                });
            },

            "peer-entity-added": function (entity) {
                var type = entity.type;

                if ((type === 'hero-jet') || (type === 'hero-mindy')) {
                    if (!this.primary) { // first time
                        this.addEventListener('handle-logic', handleLogic.bind(this));
                        this.last = Vector.setUp(entity.position);
                        this.lastPrimary = Vector.setUp(entity.position);
                    }
                    this.primary = entity;
                }
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.offset.recycle();
                if (this.last) {
                    this.last.recycle();
                    this.lastPrimary.recycle();
                }
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Vector = include('platypus.Vector'),
        processDirection = function (direction) {
            return function (state) {
                this[direction] = !state || (state.pressed !== false);
            };
        };
    
    return platypus.createComponentClass({
        id: 'HeroMovement',
        properties: {
            "maxVX": 0.3,
            "initialVX": 0.1,
            "aX": 0.001,
            "airAX": 0.0015,
            "airFriction": 0.001,
            "groundFriction": 0.003,
            "maxVY": 1,
            "initialVY": -0.8,
            "lowGravity": 0.0015,
            "highGravity": 0.0025,
            "jumpEndDampen": 0.75
        },

        initialize: function () {
            this.v = Vector.setUp(0, 0);

            this.jumpReleased = true;
            this.jumpWindow = 75;
            this.canJump = 0;

            if (typeof this.owner.headingRight === "undefined" || this.owner.headingRight) {
                this.owner.headingRight = true;
            } else {
                this.owner.headingRight = false;
                this.owner.triggerEvent('transform', 'horizontal');
            }
        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;

                if (this.right) {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x < this.initialVX) {
                            this.v.x = this.initialVX;
                        } else {
                            this.v.x += this.aX * delta;
                        }
                    } else {
                        this.v.x += this.airAX * delta;
                    }
                    
                } else if (this.left) {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x > -this.initialVX) {
                            this.v.x = -this.initialVX;
                        } else {
                            this.v.x += -this.aX * delta;
                        }
                    } else {
                        this.v.x += -this.airAX * delta;
                    }
                } else  {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x > 0) {
                            this.v.x = Math.max(this.v.x - (this.groundFriction * delta), 0);
                        } else if (this.v.x < 0) {
                            this.v.x = Math.min(this.v.x + (this.groundFriction * delta), 0);
                        }
                    } else {
                        if (this.v.x > 0) {
                            this.v.x = Math.max(this.v.x - (this.airFriction * delta), 0);
                        } else if (this.v.x < 0) {
                            this.v.x = Math.min(this.v.x + (this.airFriction * delta), 0);
                        }
                    }
                }

                if (this.owner.state.get('on-ground')) {
                    this.canJump = this.jumpWindow;
                }

                if (this.canJump) {
                    if (this.jump) {
                        this.v.y = this.initialVY;
                        this.canJump = 0;
                        //this.owner.state.set('on-ground', false);
                    } else {
                        this.v.y += delta * this.highGravity;
                    }
                } else {
                    if (!this.jumpReleased) {
                        this.v.y += delta * this.lowGravity;
                        if (this.v.y >= 0) {
                            this.jumpReleased = true;
                        }
                    } else {
                        if (this.dampenJump) {
                            if (this.v.y < 0) {
                                this.v.y *= this.jumpEndDampen;
                            }
                        }

                        this.v.y += delta * this.highGravity;
                    }
                }

                if (Math.abs(this.v.x) > this.maxVX) {
                    this.v.x = Math.max(this.v.x, -this.maxVX);
                    this.v.x = Math.min(this.v.x, this.maxVX);
                }

                if (Math.abs(this.v.y) > this.maxVY) {
                    this.v.y = Math.max(this.v.y, -this.maxVY);
                    this.v.y = Math.min(this.v.y, this.maxVY);
                }

                if (this.owner.headingRight && this.v.x < 0) {
                    this.owner.headingRight = false;
                    this.owner.triggerEvent('transform', 'horizontal');
                } else if (!this.owner.headingRight && this.v.x > 0) {
                    this.owner.headingRight = true;
                    this.owner.triggerEvent('transform', 'horizontal');
                }

                this.owner.x += this.v.x * delta;
                this.owner.y += this.v.y * delta;

                this.dampenJump = false;
                this.owner.state.set('on-ground', false);
                this.canJump = Math.max(this.canJump - delta, 0);
            },

            "go-left": processDirection ("left"),
            "go-right": processDirection ("right"),
            "jump": function (state) {
                if (state.pressed) {
                    this.jump = true;
                    this.jumpReleased = false;
                } else {
                    this.dampenJump = true;
                    this.jumpReleased = true;
                }
            },
            "hit-solid": function (collisionInfo) {
                var direction = collisionInfo.direction;

                if (direction.y !== 0 ) {
                    this.jumpReleased = true;
                    this.v.y = 0;
                    if (direction.y === 1) {
                        this.owner.state.set('on-ground', true);
                        this.jump = false;
                    }
                }

                if (direction.x === 1 || direction.x === -1) {
                    this.v.x = 0;
                }
            },
            "stop-movement": function () {
                this.v.x = 0;
                this.v.y = 0;
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.v.recycle();
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var COUNT = 2000,
        VARIANTS = {
            "jet": ['a', 'b'],
            "mindy": ['a', 'b', 'c']
        };
    
    return platypus.createComponentClass({
        id: 'Jet2End',
        
        publicProperties: {
            condition: '',
            count: 0,
            countdown: 0,
            nextLevel: 'level-1'
        },
        
        initialize: function () {
            this.remindedPlayer = false;
            this.activeCharacter = '';
            this.firstTime = true;
            this.starTotal = 0;
            this.starCount = 0;
            this.lastCount = 0;
            this.photos = Array.setUp();
            this.noMoreAudio = false;
        },

        events: {
            "scene-live": function (data) {
                this.level = data.level;
            },

            "jet-active": function () {
                this.activeCharacter = 'jet';
            },

            "mindy-active": function () {
                this.activeCharacter = 'mindy';
            },

            "handle-logic": function (tick) {
                if ((this.countdown > 0) && !platypus.game.app.voPlayer.playing) {
                    this.countdown -= tick.delta;
                    if (this.countdown < 0) {
                        this.countdown = 0;
                    }
                }
                if (this.condition) {
                    if (!this.remindedPlayer && (this.count === 1)) {
                        this.remindedPlayer = true;
                        platypus.game.app.setTimeout(this.tryToPlay.bind(this, 'tutorial-jet-2-photo'), 2000);
                        this.sunspot = this.owner.parent.addEntity({
                            type: "sunspot-intro",
                            properties: {
                                "x": this.owner.x + 96,
                                "y": this.owner.y,
                                "z": this.owner.z,
                                "scaleX": -1
                            }
                        });
                    }
                } else {
                    if (this.count !== this.lastCount) {
                        this.lastCount = this.count;
                        platypus.game.app.setTimeout(this.tryToPlay.bind(this, 'found-' + this.count + '-' + this.level), 4000);
                    } else if (!this.remindedPlayer && (this.count === 3)) {
                        this.remindedPlayer = true;
                        platypus.game.app.setTimeout(this.tryToPlay.bind(this, 'tutorial-jet-2-photos'), 6000);
                        this.sunspot = this.owner.parent.addEntity({
                            type: "sunspot-intro",
                            properties: {
                                "x": this.owner.x + 96,
                                "y": this.owner.y,
                                "z": this.owner.z,
                                "scaleX": -1
                            }
                        });
                    }
                }
            },

            "end-game": function (collision) {
                var owner = this.owner,
                    parent = owner.parent,
                    userData = parent.parent.app.userData,
                    variants = VARIANTS[this.activeCharacter];

                if (!this.countdown) {
                    if (this.condition) {
                        if (this.condition === 'mini-2') {
                            if (this.count === 1) {
                                owner.triggerEvent('level-complete-alt');
                                owner.triggerEvent('mini-2');
                                this.noMoreAudio = true;
                            } else {
                                this.owner.triggerEvent('game-find-1' + variants[(variants.length * Math.random()) >> 0] + '-' + this.activeCharacter + '-2');
                            }
                            this.countdown = COUNT;
                        } else if (this.condition === 'mini-3') {
                            if (this.count === 1) {
                                userData.write('JetTutorial', true);
                                owner.triggerEvent('level-complete-alt');
                                owner.triggerEvent('mini-3');
                                this.noMoreAudio = true;
                            } else {
                                this.owner.triggerEvent('game-find-1' + variants[(variants.length * Math.random()) >> 0] + '-' + this.activeCharacter + '-2');
                            }
                            this.countdown = COUNT;
                        } else {
                            owner.triggerEvent('level-complete-alt');
                            owner.triggerEvent(this.condition);
                            this.countdown = COUNT;
                            this.noMoreAudio = true;
                        }
                    } else { // defaults to requiring 3 photos
                        if (!this.noMoreAudio && (this.count === 3)) {
                            owner.triggerEvent('hide-photos');
                            owner.triggerEvent('level-complete');
                            owner.triggerEvent('game-found-all-' + this.activeCharacter);
                            this.noMoreAudio = true;
                            userData.write('Level', this.nextLevel);
                            userData.read('LevelsUnlocked', function (next, data) {
                                data = data || {'level-1': 1};
                                data[next] = 1;
                                userData.write('LevelsUnlocked', data);
                            }.bind(null, this.nextLevel));
                            this.owner.triggerEvent('update-star-data');
                        } else if (this.firstTime) {
                            this.owner.triggerEvent('game-find-' + (3 - this.count) + variants[(variants.length * Math.random()) >> 0] + '-' + this.activeCharacter);
                            this.firstTime = false;
                        } else {
                            this.owner.triggerEvent('game-find-' + (3 - this.count) + variants[(variants.length * Math.random()) >> 0] + '-' + this.activeCharacter + '-2');
                        }
                        this.countdown = COUNT;
                    }
                }
            },

            "skipped-photo": function (photo) {
                this.count += 1;
                this.photos[0] = photo;
            },

            "found-photo-0": function (entity) {
                this.count += 1;
                this.photos[0] = entity.photo;
                this.countdown = 0;
            },

            "found-photo-1": function (entity) {
                this.count += 1;
                this.photos[1] = entity.photo;
                this.countdown = 0;
            },

            "found-photo-2": function (entity) {
                this.count += 1;
                this.photos[2] = entity.photo;
                this.countdown = 0;
            },

            "begin-transmission": function () {
                this.beam = this.owner.parent.addEntity({
                    type: "tractor-beam",
                    properties: {
                        x: this.owner.x,
                        y: this.owner.y - 64,
                        z: this.owner.z + 1
                    }
                });
            },

            "spawn": function () {
                if (this.photos.length) {
                    this.owner.parent.addEntity({
                        type: "photo-transmission",
                        properties: {
                            photo: this.photos.greenSplice(0),
                            x: this.owner.x,
                            y: this.owner.y - 192,
                            z: this.owner.z + 2
                        }
                    });
                }
            },

            "whats-next": function () {
                if (this.beam) {
                    this.owner.parent.removeEntity(this.beam);
                    this.beam = null;
                }
            },

            "freeze": (function () {
                var
                    handleEnding = function () {
                        if (platypus.game.app.voPlayer.playing) {
                            platypus.game.app.setTimeout(handleEnding.bind(this), 500);
                            return;
                        }

                        switch (this.condition) {
                        case "mini-1":
                            this.owner.triggerEvent('new-scene', {
                                "persistentData": {"level": "mini-2"},
                                "scene": "scene-tut-one"
                            });
                            break;
                        case "mini-2":
                            this.owner.triggerEvent('new-scene', {
                                "persistentData": {
                                    "level": "level-1",
                                    "skip": "level-1-photos-a"
                                },
                                "scene": "scene-level"
                            });
                            break;
                        case "mini-3":
                            this.owner.triggerEvent('new-scene', {
                                "persistentData": {
                                    "level": "level-3",
                                    "skip": "level-3-photos-a"
                                },
                                "scene": "scene-level"
                            });
                            break;
                        default:
                            this.owner.triggerEvent('new-scene');
                            break;
                        }
                    };

                return function () {
                    platypus.game.app.setTimeout(handleEnding.bind(this), 2000); // Have to use this because this.owner's logic is paused indefinitely on the next line. - DDD
                    this.owner.parent.triggerEvent('pause-all');
                };
            }())
        },
        
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.photos.recycle();
            },

            tryToPlay: function (vo) {
                var app = platypus.game.app;

                if (this.noMoreAudio && (vo.indexOf('tutorial-jet-2-photo') >= 0)) { // This audio comes in after some time if the player forgets the next step. This ensures it's not played once the player reaches Jet 2 with all the pictures.
                    return;
                }

                if (!app.voPlayer.playing) {
                    this.owner.triggerEvent(vo);
                } else {
                    app.setTimeout(this.tryToPlay.bind(this, vo), 1000);
                }
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    var SCALE_STEP = 150,
        HAND_REACH_DISTANCE = 3.5 * 64,
        GRAB_TIME = 100;
        //HAND_REACH_HORIZONTAL = 3.5 * 64,
        //HAND_REACH_VERTICAL = 3.5 * 64;
    return platypus.createComponentClass({
        
        id: 'JetHand',
        
        initialize: function () {
            this.initial = {
                "horizontal": {
                    "x": 0,
                    "y": -64,
                    "minX": 32
                },
                "vertical": {
                    "x": 0,
                    "y": -64
                }
            };
            this.initialOffsetX = 24;
            this.initialOffsetY = 24;
            this.offsetX = 0;
            this.offsetY = 0;
            this.reachHorizontal = true;
            this.reaching = false;
            this.reachComplete = false;
            this.step = 0;
            this.pulling = false;
            this.pullingWhat = null;
            this.pullInfo = {
                "position": 0,
                "direction": 1
            };
            //this.repositionVector = platypus.Vector.setUp(0, 0);
            this.relocateMessage = {
                "unstick": false,
                "relative": false,
                "position": platypus.Vector.setUp(0, 0, 0)
            };

            this.reachDistance = 0;
            this.grabTimer = 0;

            this.facing = 1;
            this.stopReachCollision = false;

            this.owner.opacity = 0;
        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta,
                    bodyToHand = 0;
                if (this.owner.jet.headingRight && this.facing === -1) {
                    this.facing = 1;
                    this.owner.triggerEvent('transform', 'horizontal');
                } else if (!this.owner.jet.headingRight && this.facing === 1) {
                    this.facing = -1;
                    this.owner.triggerEvent('transform', 'horizontal');
                }

                if (this.freeze) {
                    return;
                } else if (this.owner.state.get('grab')) {
                    this.grabTimer -= delta;
                    if (this.grabTimer <= 0) {
                        this.owner.state.set('grab', false);
                    }
                } else if (this.reaching && !this.reachComplete) {
                    this.step += delta / SCALE_STEP;
                    if (this.step >= Math.PI) {
                        this.step = 0;
                        //SHUT THINGS DOWN!
                        this.reachComplete = true;
                    }

                    if (this.reachHorizontal) {
                        this.owner.x = this.owner.jet.x + (this.facing * (this.initial.horizontal.x + (Math.sin(this.step) * this.reachDistance)));
                        this.owner.y = this.owner.jet.y + this.initial.horizontal.y;
                    } else {
                        this.owner.x = this.owner.jet.x + (this.facing * this.initial.vertical.x);
                        this.owner.y = this.owner.jet.y + this.initial.vertical.y + (Math.sin(this.step) * -this.reachDistance);
                    }

                    if (this.pulling && this.pullingWhat) {
                        bodyToHand = Math.abs(this.owner.jet.x - this.owner.x);
                        if ( bodyToHand > this.initial.horizontal.minX) {
                            if (this.facing === 1) {
                                this.pullInfo.position = this.owner.x + (this.owner.width / 2);
                            } else {
                                this.pullInfo.position = this.owner.x - (this.owner.width / 2);
                            }

                            this.pullInfo.direction = this.facing;
                            this.pullingWhat.triggerEvent('pull', this.pullInfo);
                        } else if (this.step >= (Math.PI / 2) && bodyToHand <= this.initial.horizontal.minX) {

                            if (Math.abs(this.pullingWhat.x - this.owner.jet.x) > this.initial.horizontal.minX + (this.owner.width / 2) + (this.pullingWhat.width / 2)) {
                                if (this.facing === 1) {
                                    this.pullInfo.position = this.owner.jet.x + this.initial.horizontal.minX + (this.owner.width / 2);
                                } else {
                                    this.pullInfo.position = this.owner.jet.x - this.initial.horizontal.minX - (this.owner.width / 2);
                                }

                                this.pullInfo.direction = this.facing;
                                this.pullingWhat.triggerEvent('pull', this.pullInfo);
                            }

                            this.pulling = false;
                            this.pullingWhat = null;
                        }
                    }
                } else if (this.reachComplete) {
                    this.owner.triggerEvent('end-reach');
                } else {
                    this.owner.x = this.owner.jet.x + (this.facing * this.initial.horizontal.x);
                    this.owner.y = this.owner.jet.y + this.initial.horizontal.y;
                }
            },
            "handle-post-collision-logic": function (tick) {
                if (this.stopReachCollision) {
                    this.stopReachCollision = false;
                    this.owner.triggerEvent('collide-off');
                }
            },
            "reach": function (isVertical) {
                this.step = 0;
                this.reachHorizontal = !isVertical;
                this.reaching = true;
                this.owner.opacity = 1;
                

                this.reachDistance = HAND_REACH_DISTANCE;

                if (this.owner.jet.headingRight && this.facing === -1) {
                    this.facing = 1;
                    this.owner.triggerEvent('transform', 'horizontal');
                } else if (!this.owner.jet.headingRight && this.facing === 1) {
                    this.facing = -1;
                    this.owner.triggerEvent('transform', 'horizontal');
                }

                if (this.reachHorizontal) {
                    this.owner.x = this.owner.jet.x + (this.facing * this.initial.horizontal.x );
                    this.owner.y = this.owner.jet.y + this.initial.horizontal.y;
                } else {
                    this.owner.x = this.owner.jet.x + (this.facing * this.initial.vertical.x);
                    this.owner.y = this.owner.jet.y + this.initial.vertical.y;
                }

                this.relocateMessage.position.set(this.owner.x, this.owner.y, this.owner.z);
                this.owner.triggerEvent('relocate-entity', this.relocateMessage);
                this.owner.triggerEvent('collide-on');
            },
            "end-reach": function () {
                this.freeze = false;
                this.reachComplete = false;
                this.reaching = false;
                this.pulling = false;
                this.pullingWhat = null;
                this.stopReachCollision = true;
                this.owner.opacity = 0;
                this.owner.state.set('grab', false);
                this.owner.jet.triggerEvent('reach-complete');
            },
            "pull-jet-to": function (collisionData) {
                if (this.reaching) {
                    this.owner.state.set('grab', true);
                    this.grabTimer = 0;
                    this.freeze = true;
                    this.owner.jet.triggerEvent('pull-jet-to', this.reachHorizontal);
                }
            },
            "pull-to-jet": function (collisionData) {
                if (!this.pulling) {
                    if (this.reaching && this.reachHorizontal) {
                        this.pullingWhat = collisionData.entity;
                        this.owner.triggerEvent('collide-off');

                        this.pulling = true;
                        this.owner.state.set('grab', true);
                        this.grabTimer = GRAB_TIME;

                        this.step = Math.PI / 2;
                        if (this.facing === 1) {
                            this.reachDistance = this.owner.x - (this.owner.jet.x + this.initial.horizontal.x);
                        } else {
                            this.reachDistance = -(this.owner.x - (this.owner.jet.x - this.initial.horizontal.x));
                        }
                    }
                }

            }

        },
        
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.pullingWhat = null;
                
                this.relocateMessage.position.recycle();
                this.relocateMessage = null;
            }
        }
    });
}());
/*global include, platypus */
(function () {
    'use strict';

    var Vector = include('platypus.Vector'),
        processDirection = function (direction) {
            return function (state) {
                if (this.owner.paused) {
                    this[direction] = false;
                } else {
                    this[direction] = !state || (state.pressed !== false);
                }
            };
        };
    
    return platypus.createComponentClass({
        id: 'JetMovement',
        properties: {
            "maxVX": 0.3,
            "initialVX": 0.1,
            "aX": 0.001,
            "airAX": 0.0015,
            "airFriction": 0.001,
            "groundFriction": 0.003,
            "maxVY": 1,
            "initialVY": -0.8,
            "lowGravity": 0.0015,
            "highGravity": 0.0025,
            "jumpEndDampen": 0.75
        },
        
        initialize: function () {
            this.v = Vector.setUp(0, 0);
            this.rangeFinder = null;

            this.jumpReleased = true;
            this.jumpWindow = 75;
            this.canJump = 0;

            this.checkingRange = false;

            this.pullingTime = 0;
            this.closeEnough = 32;
            this.inVerticalReachZone = false;
            this.verticalSpeed = 0;
            this.verticalSpeedSet = [-0.02, -0.02, -0.02, -0.02333, -0.03, -0.03666, -0.04333];

            this.grabbed = null;
            this.pullJetToSpeedFactor = 0;

            if (typeof this.owner.headingRight === "undefined" || this.owner.headingRight) {
                this.owner.headingRight = true;
            } else {
                this.owner.headingRight = false;
                this.owner.triggerEvent('transform', 'horizontal');
            }

            this.owner.state.set('on-ground', false);
            this.owner.state.set('jumping', false);
            this.owner.state.set('reaching', false);
        },

        events: {
            "adopted": function () {
                this.rangeFinder = this.owner.parent.addEntity({
                    "type": "jet-range-finder",
                    "properties": {
                        "x": this.owner.x,
                        "y": this.owner.y,
                        "z": this.owner.z + 0.001,
                        "jet": this.owner
                    }
                });
            },
            "handle-logic": function (tick) {
                var delta = tick.delta;

                if (this.checkingRange) {
                    //this.v.x = 0;
                } else if (this.owner.state.get('reaching')) {
                    if (this.owner.state.get('movement-pull-jet')) {
                        if (this.owner.state.get('vertical')) {
                            //this.pullingTime -= delta / this.pullJetToSpeedFactor;
                            //this.v.y += this.pullingTime * delta;
                            this.v.y = this.verticalSpeed * delta;
                        } else {
                            if (this.owner.headingRight) {
                                this.pullingTime += delta / this.pullJetToSpeedFactor;
                            } else {
                                this.pullingTime -= delta / this.pullJetToSpeedFactor;
                            }
                            this.v.x += this.pullingTime * delta;
                        }
                    } else {
                        //this.v.x = 0;
                    }
                } else if (this.right) {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x < this.initialVX) {
                            this.v.x = this.initialVX;
                        } else {
                            this.v.x += this.aX * delta;
                        }
                    } else {
                        this.v.x += this.airAX * delta;
                    }
                    
                } else if (this.left) {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x > -this.initialVX) {
                            this.v.x = -this.initialVX;
                        } else {
                            this.v.x += -this.aX * delta;
                        }
                    } else {
                        this.v.x += -this.airAX * delta;
                    }
                } else  {
                    if (this.owner.state.get('on-ground')) {
                        if (this.v.x > 0) {
                            this.v.x = Math.max(this.v.x - (this.groundFriction * delta), 0);
                        } else if (this.v.x < 0) {
                            this.v.x = Math.min(this.v.x + (this.groundFriction * delta), 0);
                        }
                    } else {
                        if (this.v.x > 0) {
                            this.v.x = Math.max(this.v.x - (this.airFriction * delta), 0);
                        } else if (this.v.x < 0) {
                            this.v.x = Math.min(this.v.x + (this.airFriction * delta), 0);
                        }
                    }
                }


                //if (!this.owner.state.get('reaching')) {
                if (this.owner.state.get('on-ground') && !this.owner.state.get('reaching')) {
                    this.canJump = this.jumpWindow;
                }

                if (this.canJump && !this.owner.state.get('reaching')) {
                    if (this.jump) {
                        this.owner.triggerEvent('jump-sfx');
                        this.jump = false;
                        this.v.y = this.initialVY;
                        this.canJump = 0;
                        this.owner.state.set('jumping', true);
                    } else {
                        this.v.y += delta * this.highGravity;
                    }
                } else {
                    if (!this.jumpReleased && !this.owner.state.get('reaching')) {
                        this.v.y += delta * this.lowGravity;
                        if (this.v.y >= 0) {
                            this.jumpReleased = true;
                        }
                    } else {
                        if (this.dampenJump) {
                            if (this.v.y < 0) {
                                this.v.y *= this.jumpEndDampen;
                            }
                        }

                        if (!(this.owner.state.get('movement-pull-jet'))) { //Don't do this if we're getting pulled.
                            this.v.y += delta * this.highGravity;
                        }
                    }
                }
                //}
                
                if (this.v.x) {
                    this.owner.state.set('moving', true);
                } else {
                    this.owner.state.set('moving', false);
                }

                if (!this.owner.state.get('reaching') && (Math.abs(this.v.x) > this.maxVX)) {
                    this.v.x = Math.max(this.v.x, -this.maxVX);
                    this.v.x = Math.min(this.v.x, this.maxVX);
                }

                if (!this.owner.state.get('reaching') && this.v.y > this.maxVY) {
                    this.v.y = Math.min(this.v.y, this.maxVY);
                }

                if (this.owner.headingRight && this.v.x < 0) {
                    this.owner.headingRight = false;
                    this.owner.triggerEvent('transform', 'horizontal');
                } else if (!this.owner.headingRight && this.v.x > 0) {
                    this.owner.headingRight = true;
                    this.owner.triggerEvent('transform', 'horizontal');
                }

                this.owner.x += this.v.x * delta;
                
                if (this.owner.state.get('movement-pull-jet') && this.grabbed) {
                    if (this.owner.state.get('vertical')) {
                        if (this.owner.y - this.grabbed.y <= this.closeEnough) {
                            this.owner.y = this.grabbed.y + this.closeEnough;
                            this.grabbed = null;
                            this.owner.state.set('movement-pull-jet', false);
                            this.owner.state.set("pull-jet", false);
                            this.owner.state.set('vertical', false);
                            this.owner.state.set("reaching", false);
                            this.pullingTime = 0;
                            this.v.y = -1.25; //LAUNCH SPEED 1.5
                        }
                    } else {
                        if (this.owner.headingRight && (this.grabbed.x - (this.grabbed.width / 2)) - this.owner.x <= this.closeEnough) {
                            this.owner.x = (this.grabbed.x - (this.grabbed.width / 2)) - this.closeEnough;
                            this.grabbed = null;
                            this.owner.state.set('movement-pull-jet', false);
                            this.owner.state.set("pull-jet", false);
                            this.owner.state.set("reaching", false);
                            this.pullingTime = 0;
                        } else if (!this.owner.headingRight && this.owner.x - (this.grabbed.x + (this.grabbed.width / 2))  <= this.closeEnough) {
                            this.owner.x = (this.grabbed.x + (this.grabbed.width / 2)) + this.closeEnough;
                            this.grabbed = null;
                            this.owner.state.set('movement-pull-jet', false);
                            this.owner.state.set("pull-jet", false);
                            this.owner.state.set("reaching", false);
                            this.pullingTime = 0;
                        }
                    }
                }
                
                this.owner.y += this.v.y * delta;

                this.jump = false;
                this.dampenJump = false;
                this.owner.state.set('on-ground', false);
                this.canJump = Math.max(this.canJump - delta, 0);
                this.inVerticalReachZone = false;
            },

            "go-left": processDirection ("left"),
            "go-right": processDirection ("right"),
            "jump": function (state) {
                if (this.owner.paused) {
                    this.jump = false;
                    this.jumpReleased = true;
                    this.hoverPressed = false;
                    return;
                }

                if (state.pressed) {
                    this.jump = true;
                    this.jumpReleased = false;
                } else {
                    this.dampenJump = true;
                    this.jumpReleased = true;
                }
            },
            "reach": function () {
                if (!this.checkingRange && !this.owner.state.get('reaching') && this.owner.state.get('on-ground')) {
                    this.checkingRange = true;
                    this.rangeFinder.triggerEvent('check-range', !this.inVerticalReachZone);
                    this.owner.state.set('reaching', true);
                    this.owner.triggerEvent('stop-movement');
                }
            },
            "hit-solid": function (collisionInfo) {
                var direction = collisionInfo.direction;

                if (direction.y !== 0 ) {
                    this.jumpReleased = true;
                    this.v.y = 0;
                    if (direction.y === 1) {
                        this.owner.state.set('jumping', false);
                        this.owner.state.set('on-ground', true);
                        this.jump = false;
                    }
                }

                if (direction.x === 1 || direction.x === -1) {
                    this.v.x = 0;
                }
            },

            "stop-movement": function () {
                this.v.x = 0;
                this.v.y = 0;
                this.right = false;
                this.left = false;
            },
            "stop-x-movement": function () {
                this.v.x = 0;
                this.right = false;
                this.left = false;
            },

            "reach-vertical-zone": function () {
                this.inVerticalReachZone = true;
            },

            "found-something": function (rangeData) {
                var absDistance = Math.abs(rangeData.distance);

                this.checkingRange = false;

                this.grabbed = rangeData.entity;
                this.pullingTime = 0;
                
                this.owner.state.set('extend', true);
                if (this.inVerticalReachZone) {
                    this.owner.state.set('vertical', true);
                }

                this.owner.state.set('distance-1', false);
                this.owner.state.set('distance-2', false);
                this.owner.state.set('distance-3', false);
                this.owner.state.set('distance-4', false);
                this.owner.state.set('distance-5', false);
                this.owner.state.set('distance-6', false);
                this.owner.state.set('distance-7', false);

                if (this.owner.state.get('vertical')) {
                    this.pullJetToSpeedFactor = 240000 - (((absDistance - this.closeEnough) / 32) * 30000);
                } else {
                    this.pullJetToSpeedFactor = 240000 - (((absDistance - this.closeEnough) / 32) * 30000);
                }
                
                this.owner.triggerEvent('stretch');
                if (absDistance <= this.closeEnough) {
                    this.grabbed = null;
                    this.owner.state.set('distance-1', true);
                    this.verticalSpeed = this.verticalSpeedSet[0];
                    this.owner.triggerEvent('backing-up-is-better');
                } else if (absDistance <= 80) { //64
                    this.owner.state.set('distance-1', true);
                    this.verticalSpeed = this.verticalSpeedSet[0];
                    this.owner.triggerEvent('backing-up-is-better');
                } else if (absDistance <= 112) { //96
                    this.owner.state.set('distance-2', true);
                    this.verticalSpeed = this.verticalSpeedSet[1];
                } else if (absDistance <= 144) { //128
                    this.owner.state.set('distance-3', true);
                    this.verticalSpeed = this.verticalSpeedSet[2];
                } else if (absDistance <= 176) { //160
                    this.owner.state.set('distance-4', true);
                    this.verticalSpeed = this.verticalSpeedSet[3];
                } else if (absDistance <= 208) { //192
                    this.owner.state.set('distance-5', true);
                    this.verticalSpeed = this.verticalSpeedSet[4];
                } else if (absDistance <= 240) { //224
                    this.owner.state.set('distance-6', true);
                    this.verticalSpeed = this.verticalSpeedSet[5];
                } else if (absDistance > 240) { //224
                    this.owner.state.set('distance-7', true);
                    this.verticalSpeed = this.verticalSpeedSet[6];
                }
            },
            "found-nothing": function () {
                this.checkingRange = false;
            },

            "Extend": function () {
                this.armExtended();
            },
            "Idle": function () {
                if (this.owner.state.get('reaching')) {
                    this.reachAnimationComplete();
                }
            },
            "JumpUp": function () {
                if (this.owner.state.get('reaching')) {
                    this.reachAnimationComplete();
                }
            },




            /*
            "animation-ended": function (id) {
                var grabbedType = "",
                    direction = this.owner.headingRight ? 1 : -1;



                if (id === "Extend Horizontal 1" || id === "Extend Horizontal 2" || id === "Extend Horizontal 3" || id === "Extend Horizontal 4" || id === "Extend Horizontal 5" || id === "Extend Horizontal 6" || id === "Extend Horizontal 7" ||
                    id === "Extend Vertical 1" || id === "Extend Vertical 2" || id === "Extend Vertical 3" || id === "Extend Vertical 4" || id === "Extend Vertical 5" || id === "Extend Vertical 6" || id === "Extend Vertical 7" ||
                    id === "Extend Horizontal 1 Frame" || id === "Extend Horizontal 2 Frame" || id === "Extend Horizontal 3 Frame" || id === "Extend Horizontal 4 Frame" || id === "Extend Horizontal 5 Frame" || id === "Extend Horizontal 6 Frame" || id === "Extend Horizontal 7 Frame" ||
                    id === "Extend Vertical 3 Frame" || id === "Extend Vertical 4 Frame" || id === "Extend Vertical 5 Frame" || id === "Extend Vertical 6 Frame" || id === "Extend Vertical 7 Frame") {

                    this.owner.state.set("extend", false);

                    grabbedType = this.grabbed ? this.grabbed.type : "";

                    switch (grabbedType) {
                    case "pull-jet-block":
                    case "grab-vertical":
                    case "grab-horizontal":
                        this.owner.state.set("pull-jet", true);
                        this.owner.state.set("movement-pull-jet", true);
                        break;
                    default:
                        this.owner.state.set("pull-object", true);
                        if (this.grabbed) {
                            this.grabbed.triggerEvent('pull-to', this.owner.x + (direction * this.owner.width / 2));
                        }
                        break;
                    }
                } else if ((id === "Idle" || id === "JumpUp") && this.owner.state.get("reaching")) {
                    this.owner.state.set("pull-jet", false);
                    this.owner.state.set("pull-object", false);
                    this.owner.state.set("reaching", false);
                    this.owner.state.set('vertical', false);
                    this.grabbed = null;

                }
            },
			*/
            "exit-photo": function () {
                this.owner.paused = false;
            }
            

        },
        
        methods: {
            reachAnimationComplete: function () {
                this.owner.state.set("pull-jet", false);
                if (this.owner.state.get('pull-object')) {
                    this.owner.state.set("reaching", false);
                }
                this.owner.state.set("pull-object", false);
            },
            /*
            backToNeutral: function () {
                if (this.owner.state.get("reaching")) {
                    this.owner.state.set("pull-jet", false);
                    this.owner.state.set("pull-object", false);
                    this.owner.state.set("reaching", false);
                    this.owner.state.set('vertical', false);
                    this.grabbed = null;

                }
            },
            */
            armExtended: function () {
                var grabbedType = "",
                    direction = this.owner.headingRight ? 1 : -1;

                this.owner.state.set("extend", false);

                grabbedType = this.grabbed ? this.grabbed.type : "";

                switch (grabbedType) {
                case "grab-vertical":
                
                case "pull-jet-block":
                case "grab-horizontal":
                    this.owner.state.set("pull-jet", true);
                    this.owner.state.set("movement-pull-jet", true);
                    break;
                default:
                    this.owner.state.set("pull-object", true);
                    if (this.grabbed) {
                        this.owner.triggerEvent('drag');
                        this.grabbed.triggerEvent('pull-to', this.owner.x + (direction * this.owner.width / 2));
                    }
                    break;
                }
            },

            destroy: function () {
                this.grabbed = null;
                this.rangeFinder.triggerEvent('destroy-me');
                this.v.recycle();
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';
    var MOVEMENT_SPEED = 2,
        MAX_DISTANCE =   240, //64 * 4,
        MAX_DISTANCE_Y = 240; //64 * 4;
    return platypus.createComponentClass({

        id: 'JetRangeFinder',

        initialize: function () {
            this.horizontal = true;
            this.direction = 1;

            this.checking = false;
            this.doneChecking = false;

            this.rangeData = {
                "entity": null,
                "distance": 0,
                "horizontal": false
            };

            this.relocateMessage = {
                "unstick": false,
                "relative": false,
                "position": platypus.Vector.setUp(0, 0, 0)
            };

        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;
                
                if (this.checking) {
                    if (this.horizontal) {
                        this.owner.x += this.direction * MOVEMENT_SPEED * delta;
                        if (this.direction === 1 && this.owner.x >= this.owner.jet.x + (this.direction * MAX_DISTANCE)) {
                            this.owner.x = this.owner.jet.x + (this.direction * MAX_DISTANCE);
                            this.doneChecking = true;
                        } else if (this.direction === -1 && this.owner.x <= this.owner.jet.x + (this.direction * MAX_DISTANCE)) {
                            this.owner.x = this.owner.jet.x + (this.direction * MAX_DISTANCE);
                            this.doneChecking = true;

                        }
                    } else {
                        this.owner.y -= MOVEMENT_SPEED * delta;
                        if (this.owner.y <= (this.owner.jet.y - ((this.owner.jet.height * 2) / 3)) - MAX_DISTANCE_Y) {
                            this.owner.y = this.owner.jet.y - ((this.owner.jet.height * 2) / 3) - MAX_DISTANCE_Y;
                            this.doneChecking = true;
                        }
                    }
                }
            },

            "handle-post-collision-logic": function (tick) {
                if (this.doneChecking) {
                    this.doneChecking = false;
                    this.checking = false;
                    //Turn Collision Off
                    this.owner.triggerEvent('collide-off');

                    if (!this.foundSomething) {
                        this.rangeData.entity = null;
                        this.rangeData.horizontal = this.horizontal;
                        this.rangeData.distance = Infinity;   
                    }
                    this.foundSomething  = false;

                    this.owner.jet.triggerEvent('found-something', this.rangeData);
                    
                }
            },
            
            "check-range": function (isHorizontal) {
                this.horizontal = isHorizontal;
                this.checking = true;
                this.direction = this.owner.jet.headingRight ? 1 : -1;
                this.owner.x = this.owner.jet.x;
                this.owner.y = this.owner.jet.y - ((this.owner.jet.height * 2) / 3);
                
                //TURN COLLISION ON
                this.relocateMessage.position.set(this.owner.x, this.owner.y, this.owner.z);
                this.owner.triggerEvent('relocate-entity', this.relocateMessage);
                this.owner.triggerEvent('collide-on');
            },
            "found-something": function (collisionData) {
                this.doneChecking = true;
                this.foundSomething = true;
                this.rangeData.entity = collisionData.entity || null;
                this.rangeData.horizontal = this.horizontal;
                this.rangeData.distance = this.horizontal ? this.owner.x - this.owner.jet.x + (this.direction * 32) : (this.owner.jet.y - (this.owner.jet.height / 2)) - (this.owner.y - (this.owner.height / 2)) - 16;

                //this.owner.jet.triggerEvent('found-something', this.rangeData);
            }

        },
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.owner.jet = null;
                this.relocateMessage.position.recycle();
                this.relocateMessage = null;
                this.rangeData.entity = null;
                this.rangeData = null;
            }
        },
    });
}());
/*global include, platypus */
(function () {
    'use strict';

    var Vector = include('platypus.Vector'),
        joyvert = function (eventName, event) {
            var owner = this.owner,
                myLayer = owner.parent,
                otherLayer = myLayer.parent.layers[0],
                v1 = Vector.setUp(event.pixiEvent.data.global.x, event.pixiEvent.data.global.y),
                v2 = otherLayer.windowToWorld(v1);

            owner.triggerEvent(eventName, {
                entity: event.entity,
                event: event.event,
                pixiEvent: event.pixiEvent,
                x: v2.x,
                y: v2.y
            });
            v1.recycle();
            v2.recycle();
        };
    
    return platypus.createComponentClass({

        id: 'Joyverter',

        initialize: function () {
            this.state = this.owner.state;
        },

        events: {// These are messages that this component listens for
            "mousedown": function (event) {
                joyvert.call(this, 'mousedown-joyverted', event);
                this.setState(this.getState(Vector.setUp(event.x, event.y)));
            },

            "pressmove": function (event) {
                joyvert.call(this, 'pressmove-joyverted', event);
                this.setState(this.getState(Vector.setUp(event.x, event.y)));
            },

            "pressup": function (event) {
                joyvert.call(this, 'pressup-joyverted', event);
                this.setState();
            }
        },

        methods: {
            getState: (function () {
                var directions = [['east'], ['south', 'east'], ['south'], ['south', 'west'], ['west'], ['north', 'west'], ['north'], ['north', 'east']];

                return function (position) {
                    var angle = position.getAngle(),
                        segment = Math.PI / 4; // One eighth of the compass, representing North, Northeast, East, Southeast, South, Southwest, West, and Northwest.
                        
                    angle += segment / 2; // We offset the angle by one sixteenth of the compass to make a clean index into the directions array.
                    angle %= Math.PI * 2; // Now we make sure that the angle remains within one rotation.

                    return directions[(angle / segment) >> 0]; // Finally we determine which eighth of of the compass our angle is in and index into the directions array to acquire the appropriate directional label(s).
                };
            }()),

            setState: function (newState) {
                var state = this.state,
                    i = 0;

                state.set('north', false);
                state.set('east', false);
                state.set('south', false);
                state.set('west', false);

                if (newState) {
                    i = newState.length;
                    while (i--) {
                        state.set(newState[i], true);
                    }
                }
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Entity = include('platypus.Entity');

    return platypus.createComponentClass({
        
        id: 'LevelHandler',
        
        initialize: function (definition, callback) {
            var owner = this.owner,
                times = 0,
                ready = function () {
                    times--;
                    if (!times) {
                        callback();
                    }
                };

            if (platypus.supports.mobile) {
                times = 3;
                owner.addEntity("button-left", ready);
                owner.addEntity("button-right", ready);
                owner.addEntity("button-jump", ready);
            } else {
                times = 2;
                owner.addEntity("key-tutorial-x", ready);
                owner.addEntity("key-tutorial-space", ready);
                owner.addEntity("key-tutorial-arrows", ready);
            }

            return true; // notifies owner that this component is asynchronous.
        },

        events: {// These are messages that this component listens for
            "scene-loaded": function (data) {
                var characters = (platypus.game.settings.levels[data.level].properties && platypus.game.settings.levels[data.level].properties.characters) || 'both',
                    owner = this.owner;

                owner.parent.app.userData.write('Level', data.level);
                
                if (characters === 'both') {
                    owner.addEntity('button-mindy');
                    owner.addEntity('button-jet');
                    owner.triggerEventOnChildren('both!');
                }
            }
        },

        getAssetList: (function () {
            var
                unify = function (arr, entityType) {
                    var list = Entity.getAssetList({type: entityType});

                    arr.union(list);
                    list.recycle();
                };

            return function () {
                var assets = Array.setUp();
                
                unify(assets, 'button-mindy');
                unify(assets, 'button-jet');
                unify(assets, 'tractor-beam');

                if (platypus.supports.mobile) {
                    unify(assets, 'button-left');
                    unify(assets, 'button-right');
                    unify(assets, 'button-jump');
                } else {
                    unify(assets, 'key-tutorial-x');
                    unify(assets, 'key-tutorial-space');
                    unify(assets, 'key-tutorial-arrows');
                }

                return assets;
            };
        }())
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Debug = include('springroll.Debug', false),
        Entity = include('platypus.Entity'),
        LEVEL_FRAME_HEIGHT = 792,
        LEVELS = [
            "level-1",
            "level-2",
            "level-3",
            "level-4",
            "level-5",
            "level-6",
            "level-7",
            "level-8",
            "level-9",
            "level-10",
            "level-11"
        ],
        levelSorter = function (a, b) {
            return +a.replace('level-', '') - +b.replace('level-', '');
        },
        panToLevel = function (callback, data) {
            var index = this.levels.indexOf(data);

            if (index === -1) {
                index = 0;
            }
            
            this.pan(index, true);

            callback();
        };
    
    return platypus.createComponentClass({
        
        id: 'LevelPicker',
        
        initialize: function (definition, callback) {
            var userData = platypus.game.app.userData;

            this.level = "level-1";
            this.levelIndex = 0;
            this.levels = Array.setUp();
            this.completedGame = false;
            
            if (Debug) {
                this.levels.recycle();
                this.levels = LEVELS.greenSlice();
                userData.read('Level', panToLevel.bind(this, callback));
            } else {
                userData.read('LevelsUnlocked', function (levels, callback, data) {
                    var key = '';

                    if (!data) {
                        levels[0] = "level-1";
                    } else {
                        for (key in data) {
                            if (data.hasOwnProperty(key) && data[key]) {
                                levels.push(key);
                            }
                        }
                        levels.sort(levelSorter); // Just to make sure since object key orders aren't guaranteed.
                    }

                    userData.read('Level11', function (callback, data) {
                        if (data && data.starTotal) {
                            this.completedGame = true;
                        }

                        userData.read('Level', panToLevel.bind(this, callback));
                    }.bind(this, callback));
                }.bind(this, this.levels, callback));
            }

            return true;
        },

        events: {// These are messages that this component listens for
            "adopted": function () {
                var levels = this.levels,
                    i = levels.length,
                    owner = this.owner,
                    parent = owner.parent;

                parent.triggerEvent('follow', {
                    mode: 'locked',
                    entity: owner
                });

                while (i--) {
                    parent.addEntity({
                        "type": "level-preview",
                        "properties": {
                            "level": levels[i],
                            "tutorial": ((i === 0) ? "mindy" : ((i === 2) ? "jet" : "")),
                            "y": LEVEL_FRAME_HEIGHT * i,
                            "z": i + 1
                        }
                    });
                }
            },

            "scene-loaded": function (data) {
                if (data.firstLoad) {
                    if (this.completedGame) {
                        this.owner.triggerEvent('game-intro-stars');
                    } else {
                        this.owner.triggerEvent('game-intro-' + (1 + (this.levelIndex % 2)));
                    }
                } else if ((this.levelIndex === 0) && this.completedGame) { // Probably just completed Level 11.
                    this.owner.triggerEvent('game-intro-stars');
                } else {
                    this.owner.triggerEvent('game-return-' + ((this.levelIndex > 2) ? (1 + (this.levelIndex % 2)) : 1));
                }

                this.owner.parent.parent.getEntitiesByType('level-select-bg')[0].state.set('level-' + (this.levelIndex + 1), true);

                // Initial panning occurs before arrows are connected, so we run these events to make sure arrows update:
                this.triggerArrowUpdates();
            },

            "go-up": function () {
                this.pan(this.levelIndex - 1);
            },

            "go-down": function () {
                this.pan(this.levelIndex + 1);
            }
        },

        methods: {
            destroy: function () {
                this.levels.recycle();
                this.levels = null;
            },

            pan: function (index, firstTime) {
                if (this.levels[index]) {
                    this.level = this.levels[index];
                    this.levelIndex = index;
                    this.owner.y = index * LEVEL_FRAME_HEIGHT;
                    this.owner.triggerEvent('picked-level', this.level);
                    if (!firstTime) {
                        this.triggerArrowUpdates();
                    }
                }
            },

            triggerArrowUpdates: function () {
                var owner = this.owner,
                    index = this.levelIndex;

                owner.triggerEvent('in-middle');
                if (index === 0) {
                    owner.triggerEvent('hit-top');
                }
                if (index === this.levels.length - 1) {
                    owner.triggerEvent('hit-bottom');
                }
            }
        },

        getAssetList: (function () {
            var
                unify = function (arr, entity) {
                    var list = Entity.getAssetList(entity);

                    arr.union(list);
                    list.recycle();
                };

            return function () {
                // Not checking storage since it's ansynchronous. We're just pulling everything.
                var assets = Array.setUp(),
                    i = 11;
                
                while (i--) {
                    unify(assets, {type: 'level-preview', properties: {level: "level-" + (i + 1)}});
                }

                return assets;
            };
        }())
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var ACTIVE_Z = 13,
        GO_BUTTON_OFFSET = -500,
        PHOTO_OFFSET_X = 200,
        PHOTO_OFFSET_Y = 115,
        PHOTO_STEP_X = 100,
        Entity = include('platypus.Entity');

    return platypus.createComponentClass({
        
        id: 'LevelPreviewer',

        properties: {
            level: "",
            tutorial: ""
        },

        publicProperties: {
            opacity: 0.6,
            z: 0
        },
        
        initialize: function (definition, callback) {
            var levelDataTitle = this.level.replace('l', 'L').replace('-', '');

            this.children = Array.setUp();
            this.originalZ = this.z;

            platypus.game.app.userData.read('Level', function (value) {
                this.owner.triggerEvent('highlight-level', value);
            }.bind(this));
            platypus.game.app.userData.read(levelDataTitle, function (value) {
                this.levelData = value;
                if (this.tutorial === 'jet') {
                    platypus.game.app.userData.read('JetTutorial', function (value) {
                        this.tutorialPlayed = value || false;
                        callback();
                    }.bind(this));
                } else {
                    callback();
                }
            }.bind(this));

            return true;
        },

        events: {// These are messages that this component listens for
            "adopted": function () {
                var levelData = this.levelData,
                    parent = this.owner.parent,
                    state = {},
                    forceJetTutorial = (this.tutorial === 'jet') && !this.tutorialPlayed;

                state[this.level] = true;

                this.children.push(
                    parent.addEntity({
                        "type": "play-level-snapshot",
                        "properties": {
                            "spriteSheet": 'snapshot-' + this.level,
                            "x": this.owner.x + GO_BUTTON_OFFSET + 275,
                            "y": this.owner.y,
                            "z": ACTIVE_Z + 1
                        }
                    }),
                    parent.addEntity({
                        "type": "play-level",
                        "properties": {
                            "scene": (forceJetTutorial ? "scene-tut-one" : "scene-level"),
                            "persistentData": {
                                "level": (forceJetTutorial ? "mini-3" : this.level)
                            },
                            "state": state,
                            "x": this.owner.x + GO_BUTTON_OFFSET,
                            "y": this.owner.y,
                            "z": ACTIVE_Z + 2
                        }
                    }),
                    parent.addEntity({
                        "type": "play-level-title",
                        "properties": {
                            "text": platypus.game.app.config.levelNames[this.level],
                            "x": this.owner.x + GO_BUTTON_OFFSET,
                            "y": this.owner.y,
                            "z": ACTIVE_Z + 3
                        }
                    })
                );

                if (this.tutorial && !forceJetTutorial) {
                    this.children.push(
                        parent.addEntity({
                            "type": "button-tut-" + this.tutorial,
                            "properties": {
                                "x": this.owner.x + 450,
                                "y": this.owner.y - 150,
                                "z": ACTIVE_Z + 4
                            }
                        })
                    );
                }

                if (levelData) {
                    this.children.push(
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                photoSheet: this.level + '-photos',
                                photoFrame: 'a',
                                photo: this.level + '-photos-a',
                                "x": this.owner.x + PHOTO_OFFSET_X,
                                "y": this.owner.y - PHOTO_OFFSET_Y,
                                "z": ACTIVE_Z + 1
                            }
                        }),
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                photoSheet: this.level + '-photos',
                                photoFrame: 'b',
                                photo: this.level + '-photos-b',
                                "x": this.owner.x + PHOTO_OFFSET_X + PHOTO_STEP_X,
                                "y": this.owner.y,
                                "z": ACTIVE_Z + 2
                            }
                        }),
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                photoSheet: this.level + '-photos',
                                photoFrame: 'c',
                                photo: this.level + '-photos-c',
                                "x": this.owner.x + PHOTO_OFFSET_X + PHOTO_STEP_X + PHOTO_STEP_X,
                                "y": this.owner.y + PHOTO_OFFSET_Y,
                                "z": ACTIVE_Z + 3
                            }
                        }),
                        parent.addEntity({
                            "type": "level-stars",
                            "properties": {
                                "text": levelData.starCount + ' / ' + levelData.starTotal,
                                "x": this.owner.x + PHOTO_OFFSET_X - PHOTO_STEP_X,
                                "y": this.owner.y + PHOTO_OFFSET_Y * 1.25,
                                "z": ACTIVE_Z + 1
                            }
                        })
                    );
                } else {
                    this.children.push(
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                "x": this.owner.x + PHOTO_OFFSET_X,
                                "y": this.owner.y - PHOTO_OFFSET_Y,
                                "z": ACTIVE_Z + 1
                            }
                        }),
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                "x": this.owner.x + PHOTO_OFFSET_X + PHOTO_STEP_X,
                                "y": this.owner.y,
                                "z": ACTIVE_Z + 2
                            }
                        }),
                        parent.addEntity({
                            "type": "photo-placeholder",
                            "properties": {
                                "x": this.owner.x + PHOTO_OFFSET_X + PHOTO_STEP_X + PHOTO_STEP_X,
                                "y": this.owner.y + PHOTO_OFFSET_Y,
                                "z": ACTIVE_Z + 3
                            }
                        })
                    );
                }
            },

            "highlight-level": function (level) {
                if (level === this.level) {
                    this.opacity = 1;
                    this.z = ACTIVE_Z;
                } else {
                    this.opacity = 0.6;
                    this.z = this.originalZ;
                }
            }
        },

        methods: {
            destroy: function () {
                this.children.recycle();
                this.children = null;
            }
        },

        getAssetList: (function () {
            var
                unify = function (arr, entity) {
                    var list = Entity.getAssetList(entity);

                    arr.union(list);
                    list.recycle();
                };

            return function (prop0, prop1, prop2) {
                var assets = Array.setUp(),
                    level = (prop0 && prop0.level) || (prop1 && prop1.level) || (prop2 && prop2.level);
                
                unify(assets, {type: 'photo-placeholder'});
                unify(assets, {type: 'play-level-snapshot', properties: {spriteSheet: 'snapshot-' + level}});
                unify(assets, {type: 'play-level'});
                unify(assets, {type: 'play-level-title'});
                unify(assets, {type: 'level-stars'});
                unify(assets, {type: 'button-tut-mindy'});
                unify(assets, {type: 'button-tut-jet'});

                return assets;
            };
        }())
    });
}());

/* global include, platypus */
(function () {
    "use strict";

    var AABB = include('platypus.AABB');


    return platypus.createComponentClass({

        id: 'LockToBackground',
        
        properties: {
            foreground: false
        },

        constructor: function () {
            this.laxCam = null;
            this.cameraPosition = AABB.setUp();
            this.oX = this.owner.x;
            this.oY = this.owner.y;
            if (this.foreground) {
                this.owner.z = 1100;
            }
        },

        events: {
            "load": function () {
                if (this.foreground) {
                    this.owner.removeComponent(this);
                }
            },

            "camera-update": function (camera) {
                this.cameraPosition.set(camera.viewport);
            },

            "adopted": function () {
                var x = 0,
                    tilesets = this.owner.parent.getEntitiesByType('render-layer');

                for (x = 0; x < tilesets.length; x++) {
                    if (tilesets[x].layerZ === 0) {
                        this.laxCam = tilesets[x].components[0].laxCam;
                        this.lastLaxX = this.laxCam.x;
                        this.lastLaxY = this.laxCam.y;
                        return;
                    }
                }
            },
            "handle-movement": function () {
                var laxCam = this.laxCam;

                this.owner.x = this.oX + this.cameraPosition.x - laxCam.x;
                this.owner.y = this.oY + this.cameraPosition.y - laxCam.y;
            }
        },

        methods: {
            destroy: function () {
                this.cameraPosition.recycle();
                this.cameraPosition = null;
            }
        },

        publicMethods: {

        }
    });
})();

/* global include, platypus */
(function () {
    "use strict";
    
    var GRAVITY = 0.002;


    return platypus.createComponentClass({

        id: 'LogicDroplet',
        
        properties: {
            mud: false
        },

        constructor: function () {
            var s = 0.3 + Math.random() * 0.4,
                x = Math.random() - 0.5,
                y = -Math.random() - 0.5;
            
            this.owner.opacity = 1;
            this.owner.scaleX = s;
            this.owner.scaleY = s;
            
            this.owner.x += x * 50;
            this.owner.z += y * 20;
            
            this.v = {
                "x": x * 0.75,
                "y": y * 0.32
            };

            this.lifetime = 1000 + Math.random() * 500;

            if (this.mud) {
                this.owner.state.set('mud', true);
            }
        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;

                this.lifetime -= delta;
                if (this.lifetime <= 0) {
                    this.removeMe();
                    return;
                }

                this.v.y += GRAVITY * delta;

                this.owner.x += this.v.x * delta;
                this.owner.y += this.v.y * delta;
            }
        },

        methods: {
            removeMe: function () {
                this.owner.parent.removeEntity(this.owner);
            },
            destroy: function () {
            }
        },

        publicMethods: {

        }
    });
})();

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        
        id: 'Map',
        
        initialize: function () {
            this.owner.map = null;
        },

        events: {// These are messages that this component listens for
            "render-world": function (data) {
                this.owner.map = data.world;
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Vector = include('platypus.Vector'),
        processDirection = function (direction) {
            return function (state) {
                if (this.owner.paused) {
                    this[direction] = false;
                } else {
                    this[direction] = !state || (state.pressed !== false);
                }
                
            };
        };
    
    return platypus.createComponentClass({
        id: 'MindyMovement',
        properties: {
            "maxVX": 0.3,
            "maxHoverVX": 0.2,
            "initialVX": 0.1,
            "aX": 0.001,
            "airAX": 0.0015,
            "airFriction": 0.001,
            "groundFriction": 0.003,
            "maxVY": 1,
            "initialVY": -0.8,
            "lowGravity": 0.0015,
            "highGravity": 0.0025,
            "jumpEndDampen": 0.75,
            "hoverTime": 1500,
            "hoverGravity": 0.01
        },
        
        initialize: function () {
            this.v = Vector.setUp(0, 0);

            this.jumpReleased = true;
            this.jumpWindow = 75;
            this.canJump = 0;
            this.pressesThisJump = 0;
            this.hoverPressed = false;

            if (typeof this.owner.headingRight === "undefined" || this.owner.headingRight) {
                this.owner.headingRight = true;
            } else {
                this.owner.headingRight = false;
                this.owner.triggerEvent('transform', 'horizontal');
            }
        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta,
                    maxVX = 0,
                    owner = this.owner,
                    state = owner.state,
                    v = this.v;

                if (this.right) {
                    if (state.get('on-ground')) {
                        if (v.x < this.initialVX) {
                            v.x = this.initialVX;
                        } else {
                            v.x += this.aX * delta;
                        }
                    } else {
                        v.x += this.airAX * delta;
                    }
                    
                } else if (this.left) {
                    if (state.get('on-ground')) {
                        if (v.x > -this.initialVX) {
                            v.x = -this.initialVX;
                        } else {
                            v.x += -this.aX * delta;
                        }
                    } else {
                        v.x += -this.airAX * delta;
                    }
                } else  {
                    if (state.get('on-ground')) {
                        if (v.x > 0) {
                            v.x = Math.max(v.x - (this.groundFriction * delta), 0);
                        } else if (v.x < 0) {
                            v.x = Math.min(v.x + (this.groundFriction * delta), 0);
                        }
                    } else {
                        if (v.x > 0) {
                            v.x = Math.max(v.x - (this.airFriction * delta), 0);
                        } else if (v.x < 0) {
                            v.x = Math.min(v.x + (this.airFriction * delta), 0);
                        }
                    }
                }

                if (v.x) {
                    state.set('moving', true);
                } else {
                    state.set('moving', false);
                }

                if (state.get('hovering') && !this.hoverPressed) {
                    state.set('hovering', false);
                }

                if (this.canJump) {
                    if (this.jump) {
                        this.owner.triggerEvent('jump-sfx');
                        this.jump = false;
                        v.y = this.initialVY;
                        this.canJump = 0;
                    } else {
                        v.y += delta * this.highGravity;
                    }
                } else if (state.get('hovering') || (this.hoverPressed && this.hoverTimer)) {
                    if (this.hoverPressed && this.hoverTimer) {
                        state.set('hovering', true);
                    }
                    this.hoverTimer -= delta;

                    v.y = 0;
                    if (this.hoverTimer <= 0) {
                        this.hoverTimer = 0;
                        state.set('hovering', false);
                    }
                } else {
                    if (!this.jumpReleased) {
                        v.y += delta * this.lowGravity;
                        if (v.y >= 0) {
                            this.jumpReleased = true;
                            this.hoverPressed = true;
                            state.set('hovering', true);
                            
                            v.y = 0;
                        }
                    } else {
                        if (this.dampenJump) {
                            if (v.y < 0) {
                                v.y *= this.jumpEndDampen;
                            }
                        }

                        v.y += delta * this.highGravity;
                    }
                }

                if (v.y > 0) {
                    if (!state.get('falling')) {
                        state.set('falling', true);
                        state.set('rising', false);
                    }
                } else if (v.y < 0) {
                    if (!state.get('rising')) {
                        state.set('rising', true);
                        state.set('falling', false);
                    }
                } else if (state.get('falling') || state.get('rising')) {
                    state.set('rising', false);
                    state.set('falling', false);
                }

                maxVX = state.get('hovering') ? this.maxHoverVX : this.maxVX;

                if (Math.abs(v.x) > maxVX) {
                    v.x = Math.max(v.x, -maxVX);
                    v.x = Math.min(v.x, maxVX);
                }

                if (Math.abs(v.y) > this.maxVY) {
                    v.y = Math.max(v.y, -this.maxVY);
                    v.y = Math.min(v.y, this.maxVY);
                }

                if (owner.headingRight && v.x < 0) {
                    owner.headingRight = false;
                    owner.triggerEvent('transform', 'horizontal');
                } else if (!owner.headingRight && v.x > 0) {
                    owner.headingRight = true;
                    owner.triggerEvent('transform', 'horizontal');
                }

                owner.x += v.x * delta;
                owner.y += v.y * delta;

                this.jump = false;
                this.dampenJump = false;
                state.set('on-ground', false);
                this.canJump = Math.max(this.canJump - delta, 0);
            },

            "go-left": processDirection ("left"),
            "go-right": processDirection ("right"),
            "jump": function (state) {
                if (this.owner.paused) {
                    this.jump = false;
                    this.jumpReleased = true;
                    this.hoverPressed = false;
                    return;
                }

                if (state.triggered) {
                    this.pressesThisJump++;
                    if (this.pressesThisJump > 1) {
                        this.hoverPressed = true;
                    } else if (this.pressesThisJump === 1) {
                        this.jump = true;
                        this.jumpReleased = false;
                        this.hoverTimer = this.hoverTime;
                    }
                } else if (state.pressed) {
                    if (this.pressesThisJump > 1) { //THIS IS GROSS
                        this.hoverPressed = true;
                    }
                } else if (state.released) {
                    this.dampenJump = true;
                    this.jumpReleased = true;
                    this.hoverPressed = false;
                }
            },
            "hit-solid": function (collisionInfo) {
                var direction = collisionInfo.direction;

                if (direction.y !== 0 ) {
                    this.jumpReleased = true;
                    this.v.y = 0;
                    if (direction.y === 1) {
                        this.owner.state.set('on-ground', true);
                        this.owner.state.set('hovering', false);
                        this.jump = false;
                        this.pressesThisJump = 0;
                        this.canJump = this.jumpWindow;
                    }
                }

                if (direction.x === 1 || direction.x === -1) {
                    this.v.x = 0;
                }
            },

            "stop-movement": function () {
                this.v.x = 0;
                this.v.y = 0;
                this.right = false;
                this.left = false;
            },
            "stop-x-movement": function () {
                this.v.x = 0;
                this.right = false;
                this.left = false;
            },

            "exit-photo": function () {
                this.owner.paused = false;
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            destroy: function () {
                this.v.recycle();
            }
        }
    });
}());
/*global platypus */
(function () {
    'use strict';

    var BUFFER_X = 192,
        BUFFER_Y = 128;

    return platypus.createComponentClass({
        
        id: 'PhotoLoader',

        properties: {
            photo: ''
        },

        initialize: function () {
            var owner = this.owner,
                state = owner.state;

            if (owner.audible) {
                state.set('listen', true);
            } else if (owner.farAway) {
                state.set('binoculars', true);
            }

            this.here = false;
            this.wasHere = false;
            this.owner.triggerEvent('disable');

            this.owner.addComponent(new platypus.components.CollisionBasic(owner, {
                "collisionType": "hintity",
                "softCollisions": {
                    "hero": "at-feature"
                },
                "shapes": [{
                    width: this.owner.width + BUFFER_X + BUFFER_X,
                    height: this.owner.height,
                    offsetY: -this.owner.height / 2
                }]
            }));
        },

        events: {
            "load": function () {
                var components = this.owner.components,
                    i = components.length;

                while (i--) {
                    if (components[i].type === 'RenderGraphics') {
                        components[i].y = -this.owner.height / 2;
                        components[i].alpha = 0.0001;
                        break;
                    }
                }
            },

            "scene-live": function (data) {
                var hero = null;

                if (data.skip === this.photo) {
                    hero = this.owner.parent.getEntitiesByType('hero-mindy')[0] || this.owner.parent.getEntitiesByType('hero-jet')[0];
                    hero.triggerEvent('relocate-entity', this.owner);
                    this.owner.parent.removeEntity(this.owner);
                }
            },

            "handle-logic": function () {
                if (this.here !== this.wasHere) {
                    if (this.here) {
                        this.owner.opacity = 0.5;
                        this.owner.triggerEvent('enable');
                    } else {
                        this.owner.opacity = 0;
                        this.owner.triggerEvent('disable');
                    }
                    this.wasHere = this.here;
                }
                this.here = false;
            },

            "at-feature": function () {
                this.here = true;
            }
        },

        getAssetList: (function () {
            var
                getImages = function (ss, spriteSheets) {
                    if (ss) {
                        if (typeof ss === 'string') {
                            return getImages(spriteSheets[ss], spriteSheets);
                        } else if (ss.images) {
                            return ss.images.greenSlice();
                        }
                    }

                    return Array.setUp();
                };
            
            return function (component, props, defaultProps) {
                var arr = null,
                    i = 0,
                    images = null,
                    spriteSheets = platypus.game.settings.spriteSheets,
                    ssArr = (component.photo || props.photo || defaultProps.photo).split('-'),
                    ss = ssArr[0] + '-' + ssArr[1] + '-' + ssArr[2];

                if (Array.isArray(ss)) {
                    i = ss.length;
                    images = Array.setUp();
                    while (i--) {
                        arr = getImages(ss[i], spriteSheets);
                        images.union(arr);
                        arr.recycle();
                    }
                    return images;
                } else {
                    return getImages(ss, spriteSheets);
                }
            };
        }())
    });
}());


/*global platypus */
(function () {
    'use strict';
    return platypus.createComponentClass({
        
        id: 'Pullable',
        properties: {

        },
        publicProperties: {
            
        },
        
        initialize: function () {
            this.destination = 0;
            this.direction = 1;
            this.pulling = false;

            this.pullSpeed = 1;
        },
        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;
                if (this.pulling) {
                    this.owner.x += this.direction * (this.pullSpeed * delta);
                    if ((this.direction === 1  && this.owner.x >= this.destination) ||
                        (this.direction === -1 && this.owner.x <= this.destination)) {
                        this.pulling = false;
                        this.owner.x = this.destination;
                    }
                }
            },
            "pull-to": function (destination) {
                if (destination > this.owner.x) {
                    this.direction = 1;
                } else {
                    this.direction = -1;
                }

                this.destination = destination - (this.direction * (this.owner.width / 2));
                this.pulling = true;
            },
            "hit-solid": function () {
                this.pulling = false;
            }
        },
        
        methods: {
            destroy: function () {

            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    
    return platypus.createComponentClass({
        
        id: 'Relocateable',
        properties: {

        },
        
        initialize: function () {
        },
        events: {
            "reset": function () {
                this.owner.opacity = 0;
                this.owner.x = -100;
                this.owner.y = -100;
            },
            "setup-for-use": function (location) {
                this.owner.opacity = 1;
                this.owner.x = location.x;
                this.owner.y = location.y;
                this.owner.z = location.z;
            },
            "remove-me": function () {
                this.owner.parent.triggerEvent('remove-from-screening', this.owner);
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    
    return platypus.createComponentClass({
        
        id: 'RelocateMarker',
        properties: {

        },
        
        initialize: function () {
        },
        events: {
            "adopted": function () {
                this.owner.parent.triggerEvent('add-to-be-screened', this.owner);
            },
            "prepare-logic": function () {
                this.owner.parent.removeEntity(this.owner);
            }
        },
        
        methods: {
            destroy: function () {
            }
        }
    });
}());


/*global platypus */
(function () {
    'use strict';
    
    return platypus.createComponentClass({
        
        id: 'RelocateScreener',
        properties: {

        },
        
        initialize: function () {
            this.toScreen = [];
            this.availableEntities = [];
        },
        events: {
            "camera-update": function (cameraInfo) {
                var x = 0,
                    slot = null,
                    buffer = 64,
                    viewportLeft = cameraInfo.viewport.left - buffer,
                    viewportRight = cameraInfo.viewport.right + buffer,
                    toPlace = null;

                for (x = 0; x < this.toScreen.length; x++) {
                    slot = this.toScreen[x];
                    if (!slot.occupied && slot.x >= viewportLeft && slot.x <= viewportRight) {
                        if (this.availableEntities[slot.type].length === 0) {
                            this.createMore(slot.type);
                        }
                        toPlace = this.availableEntities[slot.type].pop();
                        toPlace.triggerEvent('setup-for-use', slot);
                        slot.occupied = toPlace;
                    } else if (slot.occupied && (slot.x < viewportLeft || slot.x > viewportRight)) {
                        this.clearSlot(slot);
                    }
                }
            },
            "add-to-be-screened": function (entity) {
                var ent = {
                    "type": entity.type + "-real",
                    "x": entity.x,
                    "y": entity.y,
                    "z": entity.z,
                    "occupied": null
                };
                this.toScreen.push(ent);

                if (!this.availableEntities[entity.type + "-real"]) {
                    this.availableEntities[entity.type + "-real"] = [];
                }
            },
            "remove-from-screening": function (entity) {
                var x = 0,
                    slot = null;

                for (x = 0; x < this.toScreen.length; x++) { 
                    slot = this.toScreen[x];
                    if (slot.type === entity.type && slot.x === entity.x && slot.y === entity.y) {
                        if (slot.occupied) {
                            this.clearSlot(slot);
                        }
                        this.toScreen.splice(x, 1);
                        return;
                    }
                }
            }
        },
        
        methods: {
            createMore: function (type) {
                var ent = this.owner.addEntity({
                    "type": type,
                    "properties": {
                        "x": -100,
                        "y": -100
                    }
                });

                this.availableEntities[type].push(ent);
            },
            clearSlot: function (slot) {
                var ent = slot.occupied;
                ent.triggerEvent('reset');
                this.availableEntities[ent.type].push(ent);
                slot.occupied = null;
            },
            destroy: function () {
                this.toScreen = null;
                this.availableEntities = null;
            }
        }
    });
}());

/* global include, platypus */
(function () {
    "use strict";

    var Graphics = include('PIXI.Graphics'),
        RenderContainer = include('platypus.components.RenderContainer');
    
    return platypus.createComponentClass({
        id: "RenderGraphics",
        properties: {
            offsetX: 0,
            offsetY: 0,
            offsetZ: 0
        },
        publicProperties: {
            color: 0xffff00,
            width: 100,
            height: 100
        },
        initialize: function (definition) {
            if (typeof this.color === 'string') {
                this.color = parseInt(this.color.replace('#', '0x'), 16);
            }
            
            if (!this.instructions) {
                this.beginFill(this.color).drawRect(-this.width / 2, -this.height / 2, this.width, this.height);
            }

            this.x = this.offsetX;
            this.y = this.offsetY;
            this.z = this.offsetZ;

            if (!this.owner.container) {
                this.owner.addComponent(new RenderContainer(this.owner, definition, this.addToContainer.bind(this)));
            } else {
                this.addToContainer();
            }
        },
        methods: {
            addToContainer: function () {
                var container = this.owner.container;

                container.addChild(this);
                container.reorder = true;
            },
            
            destroy: function () {
                this.owner.container.removeChild(this);
            }
        }
    }, Graphics);
}());

/* global include, platypus */
(function () {
    'use strict';

    var RenderSprite = include('platypus.components.RenderSprite');

    return platypus.createComponentClass({
        id: 'RenderModifier',

        properties: {
            modifiedRotation: 0,
            modifiedScale: 0,
            modifiedScaleX: 0,
            modifiedScaleY: 0,
            modifiedX: 0,
            modifiedY: 0
        },
        
        initialize: function (definition, callback) {
            var component = this.owner.addComponent(new RenderSprite(this.owner, definition, callback));

            component.sprite.rotation = this.modifiedRotation;
            component.sprite.x = this.modifiedX;
            component.sprite.y = this.modifiedY;
            if (this.modifiedScale) {
                component.sprite.scale.x = this.modifiedScale;
                component.sprite.scale.y = this.modifiedScale;
            }
            if (this.modifiedScaleX) {
                component.sprite.scale.x = this.modifiedScaleX;
            }
            if (this.modifiedScaleY) {
                component.sprite.scale.y = this.modifiedScaleY;
            }

            return true;
        },

        events: {
            /**
             * On receiving this message, this component removes itself from the entity. (It creates the [[RenderSprite]] component in its constructor.)
             *
             * @method 'load'
             */
            "load": function () {
                this.owner.removeComponent(this);
            }
        },

        getAssetList: function (component, props, defaultProps) {
            return RenderSprite.getAssetList(component, props, defaultProps);
        }
    });
}());

/**
 * This component is attached to entities that will appear in the game world. It renders a static or animated image. It listens for messages triggered on the entity or changes in the logical state of the entity to play a corresponding animation.
 *
 * @namespace platypus.components
 * @class RenderSprite
 * @uses platypus.Component
 */
/*global include, platypus */
(function () {
    "use strict";
    
    var RenderContainer = include('platypus.components.RenderContainer'),
        Text   = include('PIXI.Text'),
        alignments = {
            horizontal: {
                top: 0,
                middle: 0.5,
                center: 0.5,
                bottom: 1
            },
            vertical: {
                left: 0,
                middle: 0.5,
                center: 0.5,
                right: 1
            }
        };
    
    return platypus.createComponentClass({
        
        id: 'RenderText',
        
        properties: {
            /**
             * The offset of the x-axis position of the text from the entity's x-axis position.
             *
             * @property offsetX
             * @type Number
             * @default 0
             */
            offsetX: 0,

            /**
             * The offset of the y-axis position of the text from the entity's y-axis position.
             *
             * @property offsetY
             * @type Number
             * @default 0
             */
            offsetY: 0,

            /**
             * The z-index relative to other render components on the entity.
             *
             * @property offsetZ
             * @type Number
             * @default 0
             */
            offsetZ: 0,

            /**
             * Optional. Whether this object can be rotated. It's rotational angle is set by setting the this.owner.rotation value on the entity.
             *
             * @property rotate
             * @type Boolean
             * @default false
             */
            rotate: false,

            /**
             * Whether this object can be mirrored over X. To mirror it over X set the this.owner.rotation value to be > 90  and < 270.
             *
             * @property mirror
             * @type Boolean
             * @default false
             */
            mirror: false,

            /**
             * Optional. Whether this object can be flipped over Y. To flip it over Y set the this.owner.rotation to be > 180.
             *
             * @property flip
             * @type Boolean
             * @default false
             */
            flip: false,

            text: "",
            
            style: {}
        },
        
        initialize: function (definition) {
            /*
            * Text created here:
            */
            this.sprite = new Text(this.text, this.style);
            
            this.sprite.anchor.x = alignments.horizontal[this.style.align] || 0.5;
            this.sprite.anchor.y = alignments.vertical[this.style['vertical-align']] || 1;
            this.sprite.x = this.offsetX;
            this.sprite.y = this.offsetY;
            this.sprite.z = this.offsetZ;

            if (!this.owner.container) {
                this.owner.addComponent(new RenderContainer(this.owner, definition, this.addToContainer.bind(this)));
            } else {
                this.addToContainer();
            }
        },
        
        events: {
            /**
             * Sets the copy of the text
             *
             * @method 'set-text'
             * @param text {String} The text to insert.
             */
            "set-text": function (text) {
                this.sprite.text = text;
            }
        },
        
        methods: {
            addToContainer: function () {
                var container = this.owner.container;

                container.addChild(this.sprite);
                container.reorder = true;
            },
            
            destroy: function () {
                this.owner.container.removeChild(this.sprite);
                this.sprite.destroy();
                this.sprite = null;
            }
        }
    });
}());

/* global include, platypus */
(function () {
    'use strict';

    var RenderSprite = include('platypus.components.RenderSprite');

    return platypus.createComponentClass({
        id: 'RenderWater',

        properties: {
        },
        
        initialize: function () {
            this.fillWidth = 0;
            this.width = this.owner.width;
            this.left = -this.width / 2;
            this.owner.z = 975;
        },

        events: {
            "scene-live": function (data) {
                switch (data.level) {
                case "level-2":
                case "level-3":
                case "level-4":
                    this.owner.state.set('tropic', true);
                    break;
                case "level-5":
                case "level-6":
                    this.owner.state.set('desert', true);
                    break;
                case "level-7":
                case "level-8":
                case "level-9":
                    this.owner.state.set('dark', true);
                    break;
                }
            },

            "component-added": function (component) {
                if (component.type === 'RenderSprite') {
                    component.sprite.x = this.left + this.fillWidth;
                    this.fillWidth += 64;
                    if (this.fillWidth <= this.width) {
                        this.owner.addComponent(new RenderSprite(this.owner, {
                            spriteSheet: component.spriteSheet
                        }));
                    }
                }
            }
        }
    });
}());

/*global console, include, platypus */
(function () {
    'use strict';

    var FLIP_SCALE = -0.38,
        Entity = include('platypus.Entity'),
        Tween = include('createjs.Tween');

    return platypus.createComponentClass({
        
        id: 'ShowPhoto',
        
        properties: {
            photoSheet: '',
            photoFrame: '',
            vo: '',
            photo: '',
            transmit: false
        },

        initialize: function () {
            this.photos = platypus.game.app.config.photos;
            if (this.photo && typeof this.photo === 'string') {
                this.setPhoto(this.photo);
                this.displayPhoto(false);
            }
        },

        events: {// These are messages that this component listens for
            "scene-live": function (data) {
                if (data.skip && (this.owner.type === 'photo-0')) {
                    this.setPhoto(data.skip);
                    this.displayPhoto(true);
                    this.owner.triggerEvent('skip-photo', data.skip);
                }
            },

            "take-photo": function (target) {
                this.setPhoto(target.photo);
                this.showPhoto(this.photoData);
                this.displayPhoto(true);
                this.owner.parent.addEntity('flash');
            },

            "show-photo": function () {
                this.showPhoto(this.photoData);
                this.owner.triggerEvent(this.vo);
            },

            "adopted": function () {
                if (this.transmit) {
                    Tween.get(this.owner)
                        .wait(1050)
                        .to({
                            scaleX: 0.01
                        }, 100);
                    Tween.get(this.owner)
                        .to({
                            opacity: 1
                        }, 250)
                        .wait(800)
                        .to({
                            y: this.owner.y - 960,
                            scaleY: 10,
                            z: this.owner.z - 10
                        }, 300)
                        .to({
                            scaleY: 0.3
                        }, 300)
                        .call(function () {
                            if (this.owner.parent && this.owner.parent.removeEntity) {
                                this.owner.parent.removeEntity(this.owner);
                            }
                        }.bind(this));
                }
            }
        },

        methods: {
            displayPhoto: function (loaded) {
                var owner = this.owner,
                    components = owner.components,
                    component = null,
                    i = 0,
                    state = owner.state;

                state.set(this.photo, true);
                state.set(this.photoData['sunspot-pose'], true);
                if (this.photoData['sunspot-side'] === 'right') {
                    if (!loaded) {
                        owner.modifiedScaleX = FLIP_SCALE;
                    } else {
                        i = components.length;
                        while (i--) {
                            component = components[i];
                            if ((component.type === 'RenderSprite') && (component.spriteSheet === 'sunspot-poses')) { // Sunspot
                                component.sprite.scale.x = FLIP_SCALE;
                            }
                        }
                    }
                }
                this.owner.triggerEvent('enable');
            },

            setPhoto: function (photo) {
                var arr = photo.split('-');

                this.photoSheet = arr[0] + '-' + arr[1] + '-' + arr[2];
                this.photoFrame = arr[3];
                this.vo = 'found-' + arr[1] + arr[3];

                this.photo = photo;
                this.photoData = this.photos[photo];
            },

            showPhoto: function (photo) {
                var modX = 1;
                
                if (photo['sunspot-side'] === 'right') {
                    modX = -1;
                }

                this.owner.parent.addEntity({
                    "type": "photo",
                    "properties": {
                        "spriteSheet": this.photoSheet,
                        "animationMap": {
                            "default": this.photoFrame
                        },
                        "modifiedScaleX": modX,
                        "modifiedX": 18,
                        "modifiedY": -20,
                        "text": photo.title,
                        "rotation": 360,
                        "scaleX": 1,
                        "scaleY": 1,
                        "x": 0,
                        "y": 0,
                        "z": 30
                    }
                }).triggerEvent(photo['sunspot-pose']);

                this.owner.parent.addEntity({
                    "type": "button-exit"
                });
                if (photo.credit.indexOf('Public Domain') < 0) {
                    console.log('PHOTO CREDIT: "' + photo.title + '" by ' + photo.credit + ' - ' + photo.link);
                }
            }
        },
        
        getAssetList: function () {
            return Entity.getAssetList({
                type: 'flash'
            });
        }
    });
}());

/**
 * A simple component that keeps count of something and sends messages each time the count changes. Can also have a total. When it does it will display 'count / total'.
 *
 * @namespace platypus.components
 * @class Counter
 * @uses platypus.Component
 */
/*global include, platypus */
(function () {
    'use strict';
    
    var Data = include('platypus.Data');
    
    return platypus.createComponentClass({

        id: 'StarCounter',

        publicProperties: {
            /**
             * A total the counter is incrementing toward.
             *
             * @property total
             * @type number
             * @default 0
             */
            total: 0
        },

        initialize: function () {
            this.count = 0;
            this.lastTotal = 0;
            this.lastCount = 0;
        },

        events: {
            /**
             * Each step, this component detects whether the count has changed and triggers an 'update-content' event if so.
             *
             * @method 'handle-logic'
             */
            "handle-logic": function () {
                var update  = false,
                    msg = '';
                
                if (this.total !== this.lastTotal) {
                    this.lastTotal = this.total;
                    update = true;
                }
                
                if (this.count !== this.lastCount) {
                    this.lastCount = this.count;
                    update = true;
                }
                
                if (update) {
                    if (this.total) {
                        msg = String(this.count) + "/" + String(this.total);
                    } else {
                        msg = String(this.count);
                    }
                    
                    /**
                     * A call used to notify other components that the count or total has changed.
                     *
                     * @event 'set-text'
                     * @param text {string} String describing the current count.
                     */
                    this.owner.triggerEvent('set-text', msg);
                }
            },

            /**
             * Changes the total to the given value.
             *
             * @method 'add-star'
             * @param data.total {number} The new total value.
             */
            "add-star": function () {
                this.total += 1;
            },

            "both!": function () {
                this.owner.left = 0;
                this.owner.parent.getEntitiesByType('button-home')[0].left = 290;
            },

            /**
             * Changes the count to the given value.
             *
             * @method 'change-count'
             * @param data.count {number} The new count value.
             */
            "change-count": function (count) {
                this.count = count;
            },

            /**
             * Increments the count by 1.
             *
             * @method 'collect-star'
             */
            "collect-star": function () {
                this.count += 1;
            },

            "store-stars": function () {
                var levelDataTitle = this.level.replace('l', 'L').replace('-', '');

                this.owner.parent.parent.app.userData.write(levelDataTitle, {
                    starCount: this.count,
                    starTotal: this.total
                });
            },

            "scene-loaded": function (data) {
                this.level = data.level;
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Vector = include('platypus.Vector');
    
    return platypus.createComponentClass({
        id: 'StayInCircle',
        
        properties: {
            originX: 0,
            originY: 0
        },

        publicProperties: {
            "radius": 10
        },
        
        initialize: function () {
            this.origin = Vector.setUp(this.originX, this.originY);
            this.owner.backgroundDamper = Vector.setUp(0, 0);
        },

        events: {
            "handle-logic": function () {
                var v = Vector.setUp(this.owner.position).subtractVector(this.origin);

                if (v.magnitude(2) > this.radius) {
                    v.normalize().multiply(this.radius);
                    this.updateDamper(v);
                    v.addVector(this.origin);
                    this.owner.position.x = v.x;
                    this.owner.position.y = v.y;
                } else {
                    this.updateDamper(v);
                }

                v.recycle();
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            updateDamper: function (v) {
                this.owner.backgroundDamper.setVector(v).multiply(2).addVector(this.origin);
            },

            destroy: function () {
                this.owner.backgroundDamper.recycle();
                this.owner.backgroundDamper = null;
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';
    var ARC_HEIGHT = 300,
        OFF_BOTTOM = 256,
        ARC_X_RANGE = 256,
        JUMP_SCALE = 4;

    return platypus.createComponentClass({

        id: 'Sunspot',

        properties: {
            entrance: ""
        },

        initialize: function () {
            switch (this.owner.entrance) {
            case "intro":
                this.owner.state.set('balloons', true);
                this.owner.scaleX = 1;
                this.wait = 750;
                break;
            case "inflated":
                this.owner.state.set('inflated', true);
                this.owner.scaleX = -1;
                this.wait = 750;
                break;
            case "somersault":
                this.owner.state.set('somersault', true);
                this.wait = 750;
                break;
            default: //"balloons"
                this.owner.state.set('balloons', true);
                this.owner.scaleX = -1;
                this.wait = 750;
                break;
            }

            //Sunspot is intialized at the destination,
            //but we move it away from it before he actually appears onscreen.
            this.destinationX = this.owner.x;
            this.destinationY = this.owner.y;
            this.arcX = 0;
            this.arcRange = 0;

            this.owner.state.set('wave', false);
            this.owner.state.set('photo', false);
            this.owner.state.set('spin', false);
            this.owner.state.set('land', false);
            this.owner.state.set('jump', false);
            this.owner.state.set('idle', true);

            this.takingPhoto = false;

            this.cameraLeft = 0;
            this.cameraRight = 0;

            this.initialDistance = 0;
            this.timer = 0;
        },

        events: {
            "camera-update": function (cameraInfo) {
                if (this.cameraLeft === this.cameraRight) {
                    this.cameraLeft = cameraInfo.viewport.left;
                    this.cameraRight = cameraInfo.viewport.right;

                    this.owner.opacity = 1;
                    if (this.owner.state.get('balloons')) {
                        if (this.owner.entrance === 'intro') {
                            this.owner.x = this.cameraLeft - 128;
                            this.initialDistance = this.owner.x - this.destinationX;
                            this.destinationX -= this.initialDistance;
                            this.destinationY += this.initialDistance / 3;
                            this.owner.y -= this.initialDistance / 3;
                            this.timer = 2;
                        } else {
                            this.owner.x = this.cameraRight + 128;
                            this.timer = 1;
                            this.initialDistance = this.owner.x - this.destinationX;
                        }
                    } else if (this.owner.state.get('somersault')) {
                        this.owner.state.set('spin', true);
                        this.owner.y = cameraInfo.viewport.bottom + OFF_BOTTOM;
                        this.arcX = Math.sqrt(this.owner.y - (this.destinationY - ARC_HEIGHT)) + Math.sqrt(ARC_HEIGHT) + this.destinationX;
                        this.arcRange = this.arcX - this.destinationX;
                        this.owner.x = this.destinationX + ARC_X_RANGE;

                        this.owner.scaleX = -1 - JUMP_SCALE;
                        this.owner.scaleY = 1 + JUMP_SCALE;
                    } else if (this.owner.state.get('inflated')) {
                        this.owner.x = this.cameraRight + 128;
                        this.initialDistance = this.owner.x - this.destinationX;
                        this.timer = 1;
                    }
                }
            },
            "handle-logic": function (tick) {
                var distance = 0,
                    scale = 0;

                if (this.cameraLeft === this.cameraRight) {
                    return;
                }

                if (this.owner.state.get('balloons')) {
                    if (this.timer > 0) {
                        this.timer -= tick.delta / 4000;
                        if (this.timer < 0) {
                            this.timer = 0;
                        } else if (!this.owner.state.get('wave') && (((this.owner.entrance === 'intro') && (this.timer < 1.4)) || (this.timer < 0.6))) {
                            this.owner.state.set('wave', true);
                        }
                        this.owner.x = this.destinationX + (this.timer * this.initialDistance);
                        this.owner.y = this.destinationY + Math.sin(this.timer * 3 * Math.PI * 2) * 14;
                        if (this.owner.entrance === 'intro') {
                            this.owner.y -= (this.timer * this.initialDistance / 3);
                        }
                    } else if (!this.takingPhoto) {
                        if (this.owner.entrance === 'intro') {
                            this.owner.parent.removeEntity(this.owner);
                        } else {
                            this.takingPhoto = true;
                            this.owner.state.set('photo', true);
                        }
                    } else if (this.wait > 0) {
                        this.wait -= tick.delta;
                        if (this.wait <= 0) {
                            this.owner.triggerEvent('snap-photo', this.owner);
                            this.owner.parent.removeEntity(this.owner);
                        }
                    }
                } else if (this.owner.state.get('somersault')) {
                    if (this.owner.state.get('photo')) {
                        this.wait -= tick.delta;
                        if (this.wait <= 0) {
                            this.owner.triggerEvent('snap-photo', this.owner);
                            this.owner.parent.removeEntity(this.owner);
                        }
                    } else {
                        distance = this.owner.x - this.destinationX;
                        if (distance > 0) {
                            this.owner.x -= tick.delta / 3;
                            scale = ((this.owner.x - this.destinationX) / ARC_X_RANGE);
                            this.arcX = scale * this.arcRange + this.destinationX;
                            this.owner.scaleX = -1 - (scale * JUMP_SCALE);
                            this.owner.scaleY = 1 + (scale * JUMP_SCALE);

                            this.owner.y = Math.pow(this.arcX - (Math.sqrt(ARC_HEIGHT) + this.destinationX), 2) + (this.destinationY - ARC_HEIGHT);
                            //this.owner.rotation = -Math.cos(distance / 32) * 180;
                            
                        } else {
                            this.owner.x = this.destinationX;
                            this.owner.y = Math.pow(this.destinationX - (Math.sqrt(ARC_HEIGHT) + this.destinationX), 2) + (this.destinationY - ARC_HEIGHT);
                            
                            this.owner.scaleX = -1;
                            this.owner.scaleY = 1;

                            this.owner.state.set('spin', false);
                            this.owner.state.set('land', true);
                            //this.owner.y = Math.pow(this.owner.x - (Math.sqrt(ARC_HEIGHT) + this.destinationX), 2) - ARC_HEIGHT;
                        }
                    }
                } else if (this.owner.state.get('inflated')) {
                    if (this.timer > 0 && !this.owner.state.get('photo')) {
                        this.timer -= tick.delta / 4000;
                        if (this.timer < 0) {
                            this.timer = 0;
                            this.owner.state.set('photo', true);
                        }
                        this.owner.x = this.destinationX + (this.timer * this.initialDistance);
                        this.owner.y = this.destinationY + Math.sin(this.timer * 3 * Math.PI * 2) * 28;
                        this.owner.rotation = Math.sin(this.timer * 1.5 * Math.PI * 2) * 20;
                    } else {
                        this.timer -= tick.delta / 4000;
                        this.owner.rotation = Math.sin(this.timer * 1.5 * Math.PI * 2) * 20;
                        this.wait -= tick.delta;
                        if (this.wait <= 0) {
                            this.owner.triggerEvent('snap-photo', this.owner);
                            this.owner.parent.removeEntity(this.owner);
                        }
                    }
                }
                
            },
            "Idle": function () {
                if (this.owner.state.get('somersault')) {
                    this.owner.state.set('land', false);
                    this.owner.state.set('photo', true);
                }
            }
        }
    });
}());

/*global include, platypus */
(function () {
    'use strict';

    var Tween = include('createjs.Tween'),
        BINOCULAR_CAMERA_WIDTH = 490,
        BINOCULAR_CAMERA_HEIGHT = 278,
        COPY_Y = 320;

    return platypus.createComponentClass({
        
        id: 'ToolBinoculars',

        initialize: function () {
            this.goals = Array.setUp();

            this.originalCamera = null;
            this.telescopeCamera = null;
            this.actionRange = null;
            this.interactionRange = null;
            this.interactionTarget = null;

            this.leaving = false;
            this.foundGoal = null;
        },

        events: {// These are messages that this component listens for
            "animation-ended": function () {
                var state = this.owner.state;

                if (state.get('looking-fail')) {
                    state.set('looking-fail', false);
                } else if (state.get('looking-success')) {
                    state.set('looking-success', false);
                    this.setUpBinoculars();
                } else if (state.get('looking-success-over')) {
                    state.set('looking-success-over', false);
                }
            },
            
            "at-feature": function (collision) {
                if (!this.leaving) {
                    this.atFeature = collision.entity;
                }
                this.owner.state.set('at-feature', true);
            },

            "begin-swap": function () {
                this.leaving = true;
                if (this.wasAtFeature) {
                    if (!this.wasAtFeature.farAway) {
                        this.owner.triggerEvent('stop-pulse-binoculars');
                    } else {
                        this.owner.triggerEvent('stop-pulse-swapper');
                    }
                }
            },

            "handle-logic": function () {
                var owner = this.owner,
                    state = owner.state;

                if (this.wasAtFeature && !this.atFeature) {
                    if (this.wasAtFeature.farAway) {
                        owner.triggerEvent('stop-pulse-binoculars');
                    } else {
                        owner.triggerEvent('stop-pulse-swapper');
                    }
                } else if (!this.wasAtFeature && this.atFeature) {
                    if (this.atFeature.farAway) {
                        owner.triggerEvent('pulse-binoculars');
                    } else {
                        owner.triggerEvent('pulse-swapper');
                    }
                }
                if (state.get('moving') && state.get('looking-fail')) {
                    state.set('looking-fail', false);
                }
                this.wasAtFeature = this.atFeature;
                this.atFeature = null;
                state.set('at-feature', false);
            },

            "peer-entity-added": function (entity) {
                if (entity.type === 'hintity') {
                    this.goals.push(entity);
                } else if (entity.type === 'camera-guy') {
                    this.originalCamera = entity;
                }
            },

            "peer-entity-removed": function (entity) {
                if (entity.type === 'hintity') {
                    this.goals.greenSplice(this.goals.indexOf(entity));
                }
            },

            "telescope-area": function () {
                var atFeature = this.atFeature,
                    owner = this.owner,
                    state = owner.state;

                if (!atFeature || !atFeature.farAway) {
                    if (!state.get('looking-fail')) {
                        state.set('looking-fail', true);
                    }
                    if (atFeature) {
                        owner.triggerEvent('game-guide-jet');
                    }
                    return;
                } else if (this.interactionTarget) { // Already in binocular view so bail.
                    return;
                }

                if (!state.get('looking-success')) {
                    owner.triggerEvent('stop-movement');
                    owner.paused = true;
                    state.set('looking-success', true);
                }
            },

            "found-telescopic-collectible": function (collision) {
                var i = this.goals.length;

                if (!this.foundGoal) {
                    while (i--) {
                        if (collision.entity.photo === this.goals[i].photo) {
                            this.foundGoal = this.goals[i]; // need to auto snap photo here.
                            this.owner.triggerEvent('found-camera');
                            this.owner.triggerEvent(this.foundGoal.vo);
                            this.interactionTarget.triggerEvent('found-goal-target');

                            this.telescopeCamera.paused = true;
                            Tween.get(this.telescopeCamera).to({
                                x: collision.entity.x,
                                y: collision.entity.y
                            }, 300).wait(2000).call(this.tearDownBinoculars.bind(this));
                            break;
                        }
                    }
                }
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            setUpBinoculars: function () {
                var atFeature = this.atFeature,
                    binX = atFeature.binocularX || atFeature.x,
                    binY = atFeature.binocularY || COPY_Y,
                    owner = this.owner,
                    myLayer = owner.parent,
                    otherLayer = owner.parent.parent.layers[1],
                    viewport = {
                        width: BINOCULAR_CAMERA_WIDTH,
                        height: BINOCULAR_CAMERA_HEIGHT
                    };

                this.telescopeCamera = myLayer.addEntity({
                    type: 'telescope-camera-guy',
                    properties: {
                        originX: binX,
                        originY: binY,
                        x: binX + owner.x - atFeature.x,
                        y: binY + (64 * 2) // Set below center
                    }
                });

                myLayer.triggerEvent('follow', {
                    mode: 'locked',
                    entity: this.telescopeCamera
                });

                myLayer.triggerEvent('resize-camera', viewport);
                this.actionRange = myLayer.addEntity({
                    type: 'telescope-inner',
                    properties: {
                        x: binX,
                        y: binY,
                        z: 20000
                    }
                });
                this.interactionRange = otherLayer.addEntity({
                    type: 'telescope'
                });
                this.interactionTarget = otherLayer.addEntity({
                    type: 'target'
                });

                owner.triggerEvent('hide-sprite');

                if (atFeature.announceControls) {
                    if (platypus.supports.mobile) {
                        owner.triggerEvent('telescope-tutorial-mobile');
                    } else {
                        owner.triggerEvent('telescope-tutorial-desktop');
                    }
                }
            },

            tearDownBinoculars: function () {
                var owner = this.owner,
                    myLayer = owner.parent,
                    otherLayer = owner.parent.parent.layers[1],
                    viewport = {
                        width: 980,
                        height: 556
                    };

                myLayer.triggerEvent('follow', {
                    mode: 'locked',
                    entity: this.originalCamera
                });

                //owner.paused = false;
                myLayer.triggerEvent('resize-camera', viewport);

                myLayer.removeEntity(this.telescopeCamera);
                myLayer.removeEntity(this.actionRange);
                otherLayer.removeEntity(this.interactionRange);
                otherLayer.removeEntity(this.interactionTarget);

                this.telescopeCamera = null;
                this.actionRange = null;
                this.interactionRange = null;
                this.interactionTarget = null;

                owner.triggerEvent('show-sprite');

                this.spawnSunspot(this.foundGoal);
                this.foundGoal = null;
                
                owner.state.set('looking-success-over', true);
            },

            spawnSunspot: function (hintity) {
                var offsetX = 0,
                    offsetY = 0;

                if (hintity.offsetX || hintity.offsetX === 0) {
                    offsetX = hintity.offsetX;
                } else if (hintity.entrance === "inflated") {
                    offsetX = 128;
                } else if (hintity.entrance === "somersault") {
                    offsetX = 128;
                } else { //"balloons"
                    offsetX = 128;
                }

                if (hintity.offsetY || hintity.offsetY === 0) {
                    offsetY = hintity.offsetY;
                } else if (hintity.entrance === "inflated") {
                    offsetY = -hintity.height / 2;
                } else if (hintity.entrance === "somersault") {
                    offsetY = 0;
                } else { //"balloons"
                    offsetY = -hintity.height / 2;
                }

                this.owner.parent.addEntity({
                    type: "sunspot",
                    properties: {
                        entrance: hintity.entrance,
                        events: hintity.events,
                        photo: hintity.photo,
                        vo: hintity.vo,
                        x: hintity.x + offsetX,
                        y: hintity.y + offsetY,
                        z: hintity.z + 1000,
                        opacity: 0
                    }
                });
                this.owner.parent.removeEntity(hintity);
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        
        id: 'ToolObserve',

        initialize: function () {
            this.atFeature = null;
            this.goals = Array.setUp();
            this.goal = null;
            this.lastGoal = null;
            this.where = 0;
            this.jet2 = null;
            this.leaving = false;
            this.looking = Array.setUp();
        },

        events: {// These are messages that this component listens for
            "animation-ended": function () {
                var looking = this.looking,
                    owner = this.owner,
                    state = owner.state,
                    where = '';

                if (looking.length) {
                    where = looking.greenSplice(0);
                    if (where === 'point') {
                        state.set('hearing', false);
                        state.set('seeing', false);
                        if (this.goal === this.jet2) {
                            if (this.where) {
                                platypus.debug.olive((this.where < 0 ? '<-- ': '') + 'I think Jet 2 is this way.' + (this.where > 0 ? ' -->': ''));
                                owner.triggerEvent('game-guide-jet-2');
                                state.set('pointing', true);
                                looking.push('done');
                                owner.triggerEvent('face', (this.where < 0) ? 'left': 'right');
                            } else {
                                owner.paused = false;
                            }
                        } else if (this.goal.audible) {
                            if (!this.where) {
                                platypus.debug.olive('"I hear it right here." UNVEILED!!');
                                this.spawnSunspot(this.goal); // need to auto snap photo here.
                            } else {
                                platypus.debug.olive((this.where < 0 ? '<-- ': '') + 'I hear something this way.' + (this.where > 0 ? ' -->': ''));
                                owner.triggerEvent('game-guide-hear');
                                state.set('pointing', true);
                                looking.push('done');
                                owner.triggerEvent('face', (this.where < 0) ? 'left': 'right');
                            }
                        } else {
                            if (!this.where) {
                                if (!this.goal.farAway) {
                                    platypus.debug.olive('"I see it right here." UNVEILED!!');
                                    this.spawnSunspot(this.goal); // need to auto snap photo here.
                                } else {
                                    platypus.debug.olive('"I think we can see something from here. Try the binoculars!"');
                                    owner.triggerEvent('game-guide-binoculars');
                                    owner.paused = false;
                                }
                            } else {
                                if (!this.goal.farAway) {
                                    platypus.debug.olive((this.where < 0 ? '<-- ': '') + 'I see something over there.' + (this.where > 0 ? ' -->': ''));
                                    owner.triggerEvent('game-guide-see');
                                } else {
                                    platypus.debug.olive((this.where < 0 ? '<-- ': '') + 'I think we might be able to see something from over there.' + (this.where > 0 ? ' -->': ''));
                                    owner.triggerEvent('game-guide-far');
                                }
                                state.set('pointing', true);
                                looking.push('done');
                                owner.triggerEvent('face', (this.where < 0) ? 'left': 'right');
                            }
                        }
                    } else if (where === 'done') {
                        owner.state.set('hearing', false);
                        owner.state.set('seeing', false);
                        owner.state.set('pointing', false);
                        owner.paused = false;
                    } else {
                        this.owner.triggerEvent('face', where);
                    }
                }
            },

            "at-feature": function (collision) {
                if (!this.leaving) {
                    this.atFeature = collision.entity;
                }
            },

            "begin-swap": function () {
                this.leaving = true;
                if (this.wasAtFeature) {
                    if (!this.wasAtFeature.farAway) {
                        this.owner.triggerEvent('stop-pulse-observation');
                    } else {
                        this.owner.triggerEvent('stop-pulse-swapper');
                    }
                }
            },

            "handle-logic": function () {
                var goal = this.findClosest(),
                    owner = this.owner;

                if (goal.audible) {
                    owner.triggerEvent('close-to-sound');
                } else {
                    owner.triggerEvent('close-to-view');
                }

                if (this.wasAtFeature && !this.atFeature) {
                    if (!this.wasAtFeature.farAway) {
                        owner.triggerEvent('stop-pulse-observation');
                    } else {
                        owner.triggerEvent('stop-pulse-swapper');
                    }
                } else if (!this.wasAtFeature && this.atFeature) {
                    if (!this.atFeature.farAway) {
                        owner.triggerEvent('pulse-observation');
                    } else {
                        owner.triggerEvent('pulse-swapper');
                    }
                }
                this.wasAtFeature = this.atFeature;
                this.atFeature = null;
            },

            "peer-entity-added": function (entity) {
                var type = entity.type;

                if ((type === 'hintity') || (type === 'camera')) {
                    this.goals.push(entity);
                } else if (type === 'jet-2') {
                    this.jet2 = entity;
                }
            },

            "peer-entity-removed": function (entity) {
                if ((entity.type === 'hintity') || (entity.type === 'camera')) {
                    this.goals.greenSplice(this.goals.indexOf(entity));
                }
            },

            "face": function (direction) {
                var headingRight = (direction === 'right');

                if (this.owner.headingRight !== headingRight) {
                    this.owner.headingRight = headingRight;
                    this.owner.triggerEvent('transform', 'horizontal');
                }
            },

            "observe-area": function () {
                var goal = this.findClosest(),
                    owner = this.owner;
                
                if (!owner.paused) {
                    if (goal.audible) {
                        owner.state.set('hearing', true);
                    } else {
                        owner.state.set('seeing', true);
                    }
                    owner.triggerEvent('stop-movement');
                    owner.paused = true;
                    this.goal = goal;
                    if (this.atFeature === goal) {
                        this.where = 0;
                        this.looking.push('point');
                        owner.triggerEvent('face', ((goal.x - owner.x) < 0) ? 'left': 'right');
                    } else {
                        if (this.owner.headingRight) {
                            this.looking.push('left', 'point');
                        } else {
                            this.looking.push('right', 'point');
                        }
                        this.where = goal.x - owner.x;
                    }
                }
            }
        },
        
        methods: {// These are internal methods that are invoked by this component.
            findClosest: function () {
                var closest = Infinity,
                    distance = 0,
                    goal = null,
                    goals = this.goals,
                    i = goals.length,
                    owner = this.owner;
                
                if (!i) {
                    goal = this.jet2;
                } else {
                    while (i--) {
                        distance = Math.pow(goals[i].x - owner.x, 2) + Math.pow(goals[i].y - owner.y - owner.height, 2);
                        if (distance < closest) {
                            closest = distance;
                            goal = goals[i];
                        }
                    }
                }

                return goal;
            },

            spawnSunspot: function (hintity) {
                var offsetX = 0,
                    offsetY = 0;

                this.owner.triggerEvent('found-camera');
                this.owner.triggerEvent(hintity.vo);

                if (hintity.offsetX || hintity.offsetX === 0) {
                    offsetX = hintity.offsetX;
                } else if (hintity.entrance === "inflated") {
                    offsetX = 128;
                } else if (hintity.entrance === "somersault") {
                    offsetX = 128;
                } else { //"balloons"
                    offsetX = 128;
                }

                if (hintity.offsetY || hintity.offsetY === 0) {
                    offsetY = hintity.offsetY;
                } else if (hintity.entrance === "inflated") {
                    offsetY = -hintity.height / 2;
                } else if (hintity.entrance === "somersault") {
                    offsetY = 0;
                } else { //"balloons"
                    offsetY = -hintity.height / 2;
                }

                this.owner.parent.addEntity({
                    type: "sunspot",
                    properties: {
                        entrance: hintity.entrance,
                        events: hintity.events,
                        photo: hintity.photo,
                        vo: hintity.vo,
                        x: hintity.x + offsetX,
                        y: hintity.y + offsetY,
                        z: hintity.z + 1000,
                        opacity: 0
                    }
                });
                this.owner.parent.removeEntity(hintity);
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    var DEFAULT_TIME = 3000,
        OFF_SCREEN = -5000,
        restores = {
            'rotate-90': 'rotate-270',
            'rotate-270': 'rotate-90'
        };

    return platypus.createComponentClass({
        id: 'TouchTutorial',
        
        initialize: function () {
            this.owner.x = OFF_SCREEN;
            this.owner.y = OFF_SCREEN;
        },

        events: {
            "highlight-jump-button": function (time) {
                this.highlight('rotate-90', 'button-jump', 0, time);
            },

            "highlight-arm-button": function (time) {
                this.highlight('rotate-90', 'button-stretch', 0, time);
            },

            "highlight-observation": function () {
                this.highlight('diagonal-inverse', 'button-observe');
            },

            "highlight-mindy": function () {
                this.highlight('rotate-270', 'button-mindy');
            },

            "highlight-swap": function () {
                this.highlight('rotate-270', 'button-jet', 0, 1500, function () {
                    this.highlight('rotate-270', 'button-mindy', 0, 1500);
                }.bind(this));
            },

            "highlight-telescope": function () {
                this.highlight('diagonal-inverse', 'button-telescope');
            },

            "highlight-jet": function () {
                this.highlight('rotate-270', 'button-jet');
            },

            "highlight-arrow-buttons": function () {
                this.highlight('diagonal', 'button-left', 0, 1500, function () {
                    this.highlight('diagonal', 'button-right', 0, 1500);
                }.bind(this));
            }
        },

        methods: {
            highlight: (function () {
                var
                    finish = function (callback, orientation) {
                        var owner = this.owner;

                        owner.x = OFF_SCREEN;
                        owner.y = OFF_SCREEN;
                        owner.triggerEvent('transform', restores[orientation] || orientation);
                        if (callback) {
                            callback();
                        }
                    };

                return function (orientation, xOrEntity, y, time, callback) {
                    var entity = null,
                        owner = this.owner;

                    owner.triggerEvent('transform', orientation);
                    owner.triggerEvent('pointificate');
                    if (typeof xOrEntity === 'string') {
                        entity = owner.parent.getEntitiesByType(xOrEntity)[0];
                        owner.x = entity.x;
                        owner.y = entity.y;
                    } else {
                        owner.x = xOrEntity;
                        owner.y = y;
                    }
                    platypus.game.app.setTimeout(finish.bind(this, callback, orientation), time || DEFAULT_TIME);
                };
            }())
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        id: 'Tutorial',
        
        publicProperties: {
            "vo": "",
            "skip": false,
            "only": "",
            "interrupt": false,
            "feature": ""
        },
        
        initialize: function () {
            this.activeCharacter = '';
            this.alreadyPlayed = false;
        },

        events: {
            "found-photo-0": function () {
                if (this.feature === 'found-photo-0') {
                    this.alreadyPlayed = true;
                }
            },

            "found-photo-1": function () {
                if (this.feature === 'found-photo-1') {
                    this.alreadyPlayed = true;
                }
            },

            "found-photo-2": function () {
                if (this.feature === 'found-photo-2') {
                    this.alreadyPlayed = true;
                }
            },

            "scene-live": function (data) {
                if (data.skip && this.skip) {
                    if (this.skip === 'remove') {
                        this.owner.parent.removeEntity(this.owner);
                    } else {
                        this.vo = this.skip;
                    }
                }
            },

            "jet-active": function () {
                this.activeCharacter = 'jet';
            },

            "mindy-active": function () {
                this.activeCharacter = 'mindy';
            },

            "play-vo": function () {
                if (!this.alreadyPlayed && (!this.only || (this.only === this.activeCharacter))) {
                    this.alreadyPlayed = true;
                    this.owner.triggerEvent('collide-off');
                    this.tryToPlay(this.vo);
                }
            },

            "tutorial-suggest-observation": function () {
                if (this.activeCharacter === 'mindy') {
                    if (platypus.supports.mobile) {
                        this.owner.triggerEvent('suggest-observation-touch');
                    } else {
                        this.owner.triggerEvent('suggest-observation-mouse');
                    }
                } else {
                    this.owner.triggerEvent('suggest-mindy-observation');
                }
            },

            "tutorial-suggest-binoculars": function () {
                if (this.activeCharacter === 'jet') {
                    this.owner.triggerEvent('suggest-binoculars');
                } else {
                    this.owner.triggerEvent('suggest-jet-binoculars');
                }
            },

            "controls-tutorial": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-move');
                } else {
                    this.owner.triggerEvent('controls-keyboard-move');
                }
            },

            "tutorial-jump": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-jump');
                } else {
                    this.owner.triggerEvent('controls-keyboard-jump');
                }
            },

            "tutorial-mindy-jump": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-jump-mindy');
                } else {
                    this.owner.triggerEvent('controls-keyboard-jump-mindy');
                }
            },

            "tutorial-mindy-jump-alt1": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-jump-mindy-alt1');
                } else {
                    this.owner.triggerEvent('controls-keyboard-jump-mindy-alt1');
                }
            },

            "tutorial-mindy-jump-alt2": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-jump-mindy-alt2');
                } else {
                    this.owner.triggerEvent('controls-keyboard-jump-mindy-alt2');
                }
            },

            "tutorial-jet-pull-object": function () {
                if (platypus.supports.mobile) {
                    this.owner.triggerEvent('controls-touch-jet-pull-object');
                } else {
                    this.owner.triggerEvent('controls-keyboard-jet-pull-object');
                }
            }
        },

        methods: {
            tryToPlay: function () {
                var app = platypus.game.app;

                if (this.interrupt || !app.voPlayer.playing) {
                    this.owner.triggerEvent(this.vo);
                } else {
                    app.setTimeout(this.tryToPlay.bind(this), 1000);
                }
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        
        id: 'TutorialOrMenu',
        
        initialize: function () {
        },

        events: {// These are messages that this component listens for
            "released": function (data) {
                platypus.game.app.userData.read('LevelsUnlocked', function (data) {
                    if (!data || !data['level-2']) {
                        this.owner.triggerEvent('new-scene', {
                            scene: 'scene-tut-none',
                            persistentData: {
                                level: 'mini-1'
                            }
                        });
                    } else {
                        this.owner.triggerEvent('new-scene', {
                            scene: 'menu',
                            persistentData: {
                                firstLoad: true
                            }
                        });
                    }
                }.bind(this));
            }
        }
    });
}());

/**
 * This component is an example template from which a developer can create their own components. Summarize the purpose of this component here.
 *
 * @class ComponentExample
 * @uses platypus.Component
 */
/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        id: 'WeakPlatform',
        properties: {
            "onTime": 1000,
            "offTime": 3000
        },
        
        initialize: function () {
            this.owner.state.set('falling', false);
            this.owner.state.set('rising', false);
            
            this.collapsing = false;
            this.collapsed = false;

            this.timer = this.onTime;
        },

        events: {
            "handle-logic": function (tick) {
                var delta = tick.delta;

                if (this.collapsing) {
                    this.timer -= delta;
                    if (this.timer <= 0) {
                        this.collapsing = false;
                        this.collapsed = true;
                        this.timer = this.offTime;
                        this.owner.state.set('falling', true);
                        this.owner.triggerEvent('collide-off');
                        this.owner.triggerEvent('stop-wiggle');
                    }
                } else if (this.collapsed) {
                    this.timer -= delta;
                    if (this.timer <= 0) {
                        this.collapsed = false;
                        this.owner.state.set('falling', false);
                        this.owner.state.set('rising', true);
                    }
                }
            },
            "occupied-by-hero": function (collisionData) {
                if (!this.collapsing) {
                    this.collapsing = true;
                    this.timer = this.onTime;
                    this.owner.triggerEvent('start-wiggle');
                }
            },
            "animation-ended": function (id) {
                if (id === "weak-platform-two-rising") {
                    this.owner.state.set('rising', false);
                    this.owner.triggerEvent('collide-on');
                }
            },
            "reset": function () {
                this.owner.state.set('falling', false);
                this.owner.state.set('rising', false);
                this.collapsing = false;
                this.collapsed = false;
                this.timer = this.onTime;
                this.owner.triggerEvent('collide-on');
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    return platypus.createComponentClass({
        
        id: 'WhatSign',
        
        properties: {
            sign: ''
        },

        initialize: function () {
            var state = this.owner.state;

            if (this.sign) {
                state.set(this.sign, true);
            }

            if (platypus.supports.mobile) {
                state.set('mobile', true);
            } else {
                state.set('desktop', true);
            }
        }
    });
}());

/*global platypus */
(function () {
    'use strict';

    var TWO_PI = Math.PI * 2;
    
    return platypus.createComponentClass({

        id: 'Wiggle',
        properties: {
            "frequency": 1,
            "angle": 90,
            "startWiggling": true
        },
        
        initialize: function () {
            this.time = 0;
            this.wiggling = this.startWiggling;

            this.timeScale = TWO_PI * (this.frequency / 1000);
        },

        events: {
            "handle-logic": function (tick) {
                if (this.wiggling) {
                    this.time += tick.delta * this.timeScale;
                    this.owner.rotation = Math.sin(this.time) * this.angle;
                }
            },
            "start-wiggle": function () {
                this.wiggling = true;
            },
            "stop-wiggle": function () {
                this.wiggling = false;
                this.owner.rotation = 0;
            }
        }
    });
}());

//# sourceMappingURL=main.js.map